#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements linking lines between widget components."""

import wx
from pcisim.lib.dataword import DataWord
from pcisim.data import RawData
__all__ = ['Port', 'Bus', 'PortBadBits', 'BusFull', 'NoSuchDevice']

class Port(object):
    """A logical port between two components. Holds a DataWord.

    @ATTRIBS
        (DataWord) value : value to keep into the port
        (str) name : symbolic name for the port
    """
    def __init__(self, bits, name):
        """
        @PARAMS
            (int) bits : bits of the port
            (str) name : symbolic name for the port
        """
        self._bits = bits
        self.name = name
        self.value = RawData(bits)

    def get_bits(self):
        """Get port bits."""
        return self._bits

    def write(self, data):
        """Write data onto the port.

        @PARAMS
            | (int) data : data to write
            | (DataWord) data : data to write
        """
        # int to RawData conversion
        if not isinstance(data, DataWord):
            data = RawData(self._bits, data)

        # Make sure same bits
        if data.get_bits() != self._bits:
            raise PortBadBits(self, data)

        # Set data
        self.value = data

    def __str__(self):
        return "%s: %i bits value=%s" % (self._name, self._bits,
            hex(self.value.get_raw()))

class Bus(Port):
    """A one-to-many Port like object.

    @ATTRIBS
        (list) slots : available slots in the bus
        link : current active slot link
    """
    def __init__(self, bits, slots, name="bus"):
        """
        @PARAMS
            (int) bits : number of data lines in the bus
            (int) slots : maximun device number
            (str) name : optional bus name
        """
        super(Bus, self).__init__(bits, name)
        # empty slots
        self.slots = [None] * slots
        self.link = None

    def set_slot(self, dev, slot=None):
        """Set a slot in the bus, assigning a device.

        @PARAMS
            dev : device to assign the slot to
            | (int) slot : slot number
            | (None) slot : will find an available slot

        @RETURN
            (int) assigned slot number

        @RAISE
            BusFull if no slots are available
            NoSuchDevice if slot number is out of range

        @NOTES
            If given slot number is already used, the previous device is
            replaced with new one.
        """
        if slot is None:
            # Find an empty slot
            for sloti in xrange(len(self.slots)):
                if self.slots[sloti] is None:
                    self.slots[sloti] = dev
                    return sloti
            raise BusFull()
        else:
            # Place where pointed
            try:
                self.slots[slot] = dev
            except IndexError:
                raise NoSuchDevice(slot)
            return slot

    def get_slot(self, slot):
        """Get a device by slot number.

        @PARAMS
            (int) slot : slot number

        @RETURN
            device at that slot

        @RAISE
            NoSuchDevice if slot number is out of range

        @NOTES
            Can return (None) if slot is not already assigned.
        """
        try:
            dev = self.slots[slot]
        except IndexError:
            raise NoSuchDevice(slot)
        return dev

    def set_link(self, slot):
        """Set current active slot link.

        @PARAMS
            slot : slot number of the device to set

        @RETURN
            device in that slot

        @RAISE
            NoSuchDevice if slot number is out of range

        @NOTES
            Can set and return (None) if slot is not already assigned.
        """
        self.link = self.get_slot(slot)
        return self.link

    def __str__(self):
        return "Bus@%i {\n    " % self.value.get_bits() +\
            "\n    ".join(["%i - %s" % (sloti, str(self.slots[sloti]))
            for sloti in xrange(len(self.slots))]) +\
            "\n} : %s" % self.value

#--------------------------E X C E P T I O N S------------------------------

class NoSuchDevice(Exception):
    """Notifies that given device is not being found.

    @ATTRIBS
        (int) self.devid : identifier for the device
    """
    def __init__(self, devid):
        """
        @PARAMS
            (int) self.devid : identifier for the device
        """
        self.devid = devid
        super(NoSuchDevice, self).__init__()

    def __str__(self):
        return "No such device: #%i" % self.devid

class PortBadBits(Exception):
    """Notifies Port bits value mismatching.

    @ATTRIBS
        (Port) port : subject port
        (DataWord) word : subject value
    """
    def __init__(self, port, word):
        """
        @PARAMS
            (Port) port : subject port
            (DataWord) word : subject value
        """
        self.port = port
        self.word = word
        super(PortOverflow, self).__init__()

    def __str__(self):
        return "Port %s holds %i bits values but value has %i bits" % (
            self.port, self.port.get_bits(), self.word.get_bits() )

class BusFull(Exception):
    """Notifies that a Bus is full.

    @ATTRIBS
        (Bus) bus : subject Bus
    """

    def __init__(self, bus):
        """
        @PARAMS
            (Bus) bus : subject Bus
        """
        self.bus = bus
        super(BusFull, self).__init__()

    def __str__(self):
        return "Bus is full: %s" % self.bus
