#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A framework to build custom wx events."""

import wx
__all__ = [
    'PciEvent',

    #--Simulation control
    'SimulatorStart',
    'SimulatorStep',
    'SimulatorEnds',
    'SimulatorReset',

    #-- Output presentation
    'Message',
    'HighlightField',
    'HighlightPort',
    'HighlightWidget',
    'InspectPort',
    'InspectMemory',
    'InspectDevice',

    #-- Other
    'OpenConfigNotification',
    'AddressRelocation',
    'new_event'
]

def new_event():
    """Creates a new event.

    @RETURN
        (list[2]) returns the twin wxEventType, wx.PyEventBinder
    """
    evtype = wx.NewEventType()
    return (evtype, wx.PyEventBinder(evtype, 1))

class PciEvent(wx.PyCommandEvent):
    """Base class for all the PCI simulator events.

    In order to define a new event, the following class-scope variables
    must be exposed:
        EVID : (wxEventType) event specific ID
        EVT : (wx.PyEventBinder) event specific binder
    These variable can be generated in one call issuing:
        EVID, EVT = new_event()

    In order to bind a PciEvent called 'MyEvent', use the following syntax:
        obj.Bind(MyEvent.EVT, func)

    In order to send a PciEvent called 'MyEvent', use the following syntax:
        MyEvent(...).fire(widget)
    where widget is optionally sets the event object source.
    """

    def __init__(self):
        assert hasattr(self.__class__, 'EVID')
        assert hasattr(self.__class__, 'EVT')

        evtype = getattr(self.__class__, 'EVID')
        super(PciEvent, self).__init__(evtype)

    def fire(self, widget=None):
        """Generates and propagates the event.

        @PARAMS
            (wx.EvtHandler) widget :  optional origin widget

        @NOTES
            - If not widget source is specified, application WireBoard is
              used.
            - It is safe to call 'fire' multiple times for the same event
              object.
        """
        # Use instance as event builder
        evt = self.Clone()

        if widget is None:
            widget = wx.GetApp().board

        evt.SetEventObject(widget)
        evt.SetId(widget.GetId())
        # Deferred way : process the event in the future
        wx.PostEvent(widget, evt)
        # Alternative way : directly process event
        #~ widget.GetEventHandler().ProcessEvent(evt)

#-----------------------------E V E N T S-----------------------------------

class OpenConfigNotification(PciEvent):
    """Notify about config window opened."""
    EVID, EVT = new_event()

class AddressRelocation(PciEvent):
    """Informs about an address relocation took place.

    @ATTRIBS
        (int) oldaddr : the address wich was relocated
        (MemoryBlock) block : the block which contains the updated
            information
    """
    EVID, EVT = new_event()

    def __init__(self, oldaddr, block):
        """
        @PARAMS
            (int) oldaddr : the address wich was relocated
            (MemoryBlock) block : the block which contains the updated
                information
        """
        super(AddressRelocation, self).__init__()
        self.oldaddr = oldaddr
        self.block = block

class Message(PciEvent):
    """Used to show a message in the application log.

    @ATTRIBS
        (str) message : message to show
    """
    EVID, EVT = new_event()

    def __init__(self, message):
        """
        @PARAMS
            (str) message : message to show
        """
        super(Message, self).__init__()
        self.message = message

class HighlightField(PciEvent):
    """Highlights a DataField.

    @ATTRIBS
        (DataField) field : datafield to highlight
    """
    EVID, EVT = new_event()

    def __init__(self, field):
        """
        @PARAMS
            (DataField) field : datafield to highlight
        """
        super(HighlightField, self).__init__()
        self.field = field

class HighlightPort(PciEvent):
    """Highlights a Port object.

    @ATTRIBS
        (Port) port : port to highlight
        (str) mode : highlight mode, 'read' | 'write'
    """
    EVID, EVT = new_event()

    def __init__(self, port, mode):
        """
        @PARAMS
            (Port) port : port to highlight
            (str) mode : highlight mode, 'read' | 'write'
        """
        super(HighlightPort, self).__init__()
        self.port = port
        self.mode = mode
        assert mode in ('read', 'write')

class HighlightWidget(PciEvent):
    """Highlights a widget in the board.

    @ATTRIBS
        (Widget) widget : widget to highlight
    """
    EVID, EVT = new_event()

    def __init__(self, widget):
        """
        @PARAMS
            (Widget) widget : widget to highlight
        """
        super(HighlightWidget, self).__init__()
        self.widget = widget

class InspectPort(PciEvent):
    """Inspects port data in the DataView.

    @ATTRIBS
        (Port) port : port to inspect
    """
    EVID, EVT = new_event()

    def __init__(self, port):
        """
        @PARAMS
            (Port) port : port to inspect
        """
        super(InspectPort, self).__init__()
        self.port = port

class InspectMemory(PciEvent):
    """Notify to inspect the NorthBridge memory allocations in the DataView.
    """
    EVID, EVT = new_event()

class InspectDevice(PciEvent):
    """Inspects device memory in the DataView.

    @ATTRIBS
        (PciDeviceice) device to inspect
    """
    EVID, EVT = new_event()

    def __init__(self, pcidev):
        """
        @PARAMS
            (PciDeviceice) device to inspect
        """
        super(InspectDevice, self).__init__()
        self.dev = pcidev

class SimulatorStart(PciEvent):
    """Notifies the start of the simulation.

    @ATTRIBS
        (SimuAction) action : action being executed
    """
    EVID, EVT = new_event()

    def __init__(self, action):
        """
        @PARAMS
            (SimuAction) action : action being executed
        """
        self.action = action
        super(SimulatorStart, self).__init__()

class SimulatorStep(PciEvent):
    """Notifies a step into the simulator.

    @ATTRIBS
        (Step) step : step being executed
    """
    EVID, EVT = new_event()

    def __init__(self, step):
        """
        @PARAMS
            (Step) step : step being executed
        """
        super(SimulatorStep, self).__init__()
        self.step = step

class SimulatorEnds(PciEvent):
    """Notifies that the simulator has performed all the steps."""
    EVID, EVT = new_event()

class SimulatorReset(PciEvent):
    """Notifies that the simulator has been reset to begin a new action."""
    EVID, EVT = new_event()
