#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A module for unit testing in pcisim package.

Each test-able module should implement one of the following interfaces:
    _main(app) : a function with takes a wx.App parameter and returns
        a return value

    _build(parent) : a function or a wx.Window derived class which takes
        a wx.Window parent as a parameter and returns a wx.Window instance
"""

import importlib
import wx
from pcisim.widgets import WireBoard
from pcisim.core.simulator import Simulator
from pcisim.core.locals import WINDOW_MINSIZE
__all__ = ['parse_args', 'test_module']

def _usage():
    print "Usage: main.py module/to/test.py"
    exit(1)

def parse_args(argv):
    """Parse argv and call test_module function.

    @RETURN
        Tested module return value
    """
    if len(argv) != 2:
        _usage()

    modules = argv[1].split('/')

    if (len(modules[-1]) < 3) or (modules[-1][-3:] != ".py"):
        _usage()
    modules[-1] = modules[-1][:-3]
    impath = '.'.join(modules)
    module = importlib.import_module(impath, package='')
    return test_module(module)

def test_module(module):
    """Testes a module with exposed testing interface.

    @PARAMS
        module : module to test

    @RETURN
        Tested module return value
    """
    app = Simulator()
    app.frame.Hide()
    frame = wx.Frame(None, title='pcisim.'+module.__name__,
        size=WINDOW_MINSIZE)
    pan = wx.Panel(frame)

    if hasattr(module, '_build'):
        widget = module._build(pan)
    elif hasattr(module, '_main'):
        return module._main(app)
    else:
        assert 0, "No '_main' or '_build' functions to call in module"

    frame.Center()
    frame.Show()
    frame.Bind(wx.EVT_WINDOW_DESTROY, lambda ev: app.frame.Destroy())
    app.MainLoop()
    return 0
