#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

import wx
from pcisim.lib.action import Action, Step
from pcisim.lib.widget import Widget
from .events import HighlightWidget, HighlightPort, InspectPort
__all__ = ['SimuAction']

"""Implements SimuAction base class."""

class SimuAction(Action):
    """Implements additional method to automate port highlight functions.

    @PARAMS
        (WireBoard) board : points to app main board.

    On each step, a subclass *should* call 'set_subject' function to set
    active object. Subject should have at least these attributes:
        - config_port
        - data_port
        - pcibus
    """
    def __init__(self, *args, **kargs):
        """
        @PARAMS
            *args : parameters to pass to Action class
            **kargs : keyword paramters to pass to Action class

        See pcisim.lib.action for more details.
        """
        self.board = wx.GetApp().board
        self._subject = None
        super(SimuAction, self).__init__(*args, **kargs)

    def set_subject(self, subj):
        """Set action subject and highlights it.

        @PARAMS
            subj : current action subject.
        """
        # Set action subject and highlights it
        self._subject = subj
        if isinstance(subj, Widget):
            HighlightWidget(subj).fire()

    def config_out(self, data):
        """Write config onto config_port.

        The following effects are triggered:
            - port_config is set to WRITE mode
            - port_config data is written
            - port_config is dumped into the DataView

        @PARAMS
            data : data to write on the port
        """
        self._subject.port_config.write(data)
        HighlightPort(self._subject.port_config, mode='write').fire(self._subject)
        InspectPort(self._subject.port_config).fire(self._subject)

    def config_in(self):
        """Reads config from config_port

        The following effects are triggered:
            - port_config is set to READ mode
            - port_config data is retrieved
            - pcibus is set to WRITE mode
            - pcibus is written with config
            - pcibus is dumped into the DataView

        @RETURN
            the read word

        @NOTES
            only NorthBridge should call config_in
        """
        HighlightPort(self._subject.port_config, mode='read').fire(self._subject)
        data = self._subject.port_config.value
        HighlightPort(self._subject.pcibus, mode='write').fire(self._subject)
        self._subject.pcibus.write(data)
        InspectPort(self._subject.pcibus).fire(self._subject)
        return data

    def data_out(self, data):
        """Writes data to data_port

        The following effects are triggered:
            - port_data is set to WRITE mode
            - port_data is written
            - port_data is dumped into the DataView

        @PARAMS
            data : data to write on the port
        """
        self._subject.port_data.write(data)
        HighlightPort(self._subject.port_data, mode='write').fire(self._subject)
        InspectPort(self._subject.port_data).fire(self._subject)

    def data_outbus(self, data):
        """Writes data to pcibus and copies it to data_port

        The following effects are triggered:
            - pcibus is set to WRITE mode
            - pcibus is written
            - port_data is set to WRITE mode
            - port_data is written
            - port_data is dumped into the DataView

        @PARAMS
            data : data to write on the port

        @NOTES
            only NorthBridge should call data_outbus
        """
        HighlightPort(self._subject.pcibus, mode='write').fire(self._subject)
        self._subject.pcibus.write(data)
        self.data_out(data)

    def data_in(self):
        """Reads data from pcisim.core.port

        The following effects are triggered:
            - port_data is set to READ mode
            - port_data is read
            - port_data is dumped into the DataView

        @RETURN
            the read value
        """
        HighlightPort(self._subject.port_data, mode='read').fire(self._subject)
        InspectPort(self._subject.port_data).fire(self._subject)
        return self._subject.port_data.value

    def data_inbus(self):
        """Reads data from pcisim.core.port and copies value to pcibus

        The following effects are triggered:
            - port_data is set to READ mode
            - port_data is read
            - port_data is dumped into the DataView
            - pcibus is set to WRITE mode
            - pcibus is written

        @RETURN
            the read value

        @NOTES
            only NorthBridge should call data_inbus
        """
        data = self.data_in()
        HighlightPort(self._subject.pcibus, mode='write').fire(self._subject)
        self._subject.pcibus.write(data)
        return data
