#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements the main application."""

import wx
from wx.lib.scrolledpanel import ScrolledPanel
import pcisim
from pcisim.core.locals import *
from pcisim.core.events import *
from pcisim.data import ReservedMemory
from pcisim.lib.utils import get_hex
from pcisim.widgets import DataViewWindow, ActionList, ControlPanel,\
    WireBoard
__all__ = ['Simulator']

class Simulator(wx.App):
    """The simulator application.

    @ATTRIBS
        (wx.Frame) frame : main application frame
        (WireBoard) board : container for all the Widgets
    """
    def OnInit(self):
        """Builds application layout."""
        self._highligths = []    # Highlighted fields, used to reset on step
        self.frame = wx.Frame(None, title="PCI Bus Simulator",
            size=WINDOW_SIZE)
        self.frame.SetMinSize(WINDOW_MINSIZE)
        splitw_vert = wx.SplitterWindow(self.frame)
        splitw_bottom = wx.SplitterWindow(splitw_vert)
        box = ScrolledPanel(splitw_vert)
        self.board = WireBoard(box)
        self.board.populate()
        self._dataview = DataViewWindow(splitw_bottom)
        self._actlist = ActionList(splitw_bottom)
        self._ctrlpan = ControlPanel(self.frame)
        self._actlist.populate(self._ctrlpan.get_main_action())

        # Application Icon
        icon = wx.Icon(pickimage('icon.png'), wx.BITMAP_TYPE_PNG)
        self.frame.SetIcon(icon)

        # Board container for scrolling
        l = wx.BoxSizer(wx.HORIZONTAL)
        l.AddStretchSpacer()
        l.Add(self.board, 0, flag=wx.ALIGN_CENTER)
        l.AddStretchSpacer()
        box.SetSizer(l)
        box.SetupScrolling()
        box.SetBackgroundColour(wx.WHITE)

        # Split windows
        splitw_vert.SplitHorizontally(box, splitw_bottom)
        splitw_vert.SetMinimumPaneSize(ACTLIST_MINHEIGHT)
        splitw_vert.SetSashPosition(WINDOW_SIZE[1]-
            self._ctrlpan.GetBestSize()[1]-ACTLIST_MINHEIGHT-50)
        splitw_bottom.SplitVertically(self._actlist, self._dataview)
        splitw_bottom.SetMinimumPaneSize(ACTLIST_MINWIDTH)
        splitw_bottom.SetSashPosition(ACTLIST_DEFAULT_WIDTH)

        # Sizer
        vert = wx.BoxSizer(wx.VERTICAL)
        vert.Add(self._ctrlpan, flag=wx.EXPAND)
        vert.Add(splitw_vert, 1, flag=wx.EXPAND)
        self.frame.SetSizer(vert)
        self.frame.Centre()
        self.frame.Show()

        # BINDINGS
        self.Bind(SimulatorStep.EVT, self._on_simu_step)
        self.Bind(SimulatorReset.EVT, self._on_simu_reset)
        self.Bind(Message.EVT, self._show_message)
        self.Bind(OpenConfigNotification.EVT,
            lambda ev: self._ctrlpan.stop())
        self.Bind(AddressRelocation.EVT, self._on_relocation)
        self.Bind(HighlightField.EVT, self._on_field_highlight)
        self.Bind(HighlightPort.EVT, self._on_port_highlight)
        self.Bind(HighlightWidget.EVT, self._on_widget_highlight)
        self.Bind(InspectPort.EVT, self._on_port_inspect)
        self.Bind(InspectDevice.EVT, self._on_device_inspect)
        self.Bind(InspectMemory.EVT, self._on_memory_inspect)
        return True

#---------------------------P U B L I C   A P I-----------------------------

    def get_control_panel(self):
        """Get application control panel.

        @RETURN
            (ControlPanel) control panel.
        """
        return self._ctrlpan

    def get_dev_by_address(self, addr):
        """Search for a device with the specified address.

        @PARAMS
            (int) addr : address to match

        @RETURN
            | (list[2]) if found, as PCIDevice,BAR holding address
            | (None) instead
        Looks for bar at the specified address
        """
        # Truncate to 28-bits like 'address' data field
        addr = addr >> 4

        for dev in self.board.pcidevs:
            for bar in dev:
                if bar.address == addr:
                    return (dev,bar)
        return None

    def about_dialog(self):
        """Shows an about dialog."""

        # Stops simulation if running
        self._ctrlpan.stop()

        # Dialog
        about = wx.AboutDialogInfo()
        about.SetName(pcisim.PROJECT_NAME)
        about.SetVersion(pcisim.PROJECT_VERSION)
        about.SetDescription(pcisim.PROJECT_DESCRIPTION)
        about.SetLicense(pcisim.PROJECT_LICENCE)
        about.SetCopyright(pcisim.PROJECT_COPYRIGHT)
        about.SetWebSite(pcisim.PROJECT_WEBSITE)
        map(about.AddDeveloper, pcisim.PROJECT_DEVELOPERS)
        wx.AboutBox(about)

#---------------------------I N T E R N A L S-------------------------------

    def _show_message(self, evt):
        self._ctrlpan.show_message(evt.message)

    def _on_relocation(self, evt):
        # Updates device addres
        Message("\tAddress %s relocated to %s" % (get_hex(evt.oldaddr, 32),
         get_hex(evt.block.start, 32) )).fire()
        bar = self.get_dev_by_address(evt.oldaddr)[1]
        bar.lock_address(False)
        bar.set_address(evt.block.start)

    def _on_field_highlight(self, evt):
        field = evt.field
        self._highligths.append( field )
        field.highlight()

    def _on_port_highlight(self, evt):
        self.board.highlight(evt.port, evt.mode is 'read')

    def _on_widget_highlight(self, evt):
        self.board.highlight_widget(evt.widget)

    def _highlight_none(self):
        # Un-Highlight fields
        for field in self._highligths:
            field.highlight(False)
        self._highligths = []

    def _on_device_inspect(self, evt):
        self._dataview.clear()
        pcidev = evt.dev
        self._dataview.add_registers(
            pcidev.get_device_memory().get_header(), bits=True, names=True)
        self._dataview.add_registers(
            pcidev.get_device_memory().get_bars(), bits=True, header=True)
        self._dataview.done_layout()

    def _on_memory_inspect(self, evt=None):
        self._dataview.clear()
        self._dataview.add_fieldnames_row(ReservedMemory())
        self._dataview.new_line()

        # Inspect memory
        for block in self.board.bridge.meman.blocks:
            ret = self.get_dev_by_address(block.start)
            if ret is None:
                continue
            # Fill fields structure
            dev, bar = ret
            slot = ReservedMemory()
            slot.start = block.start
            slot.end = block.end
            slot.dev = dev.get_device_id()
            slot.bar = bar.barid

            self._dataview.add_word(slot)
        self._dataview.done_layout()

    def _on_port_inspect(self, evt):
        self._dataview.clear()
        self._dataview.add_word(evt.port.value, bits=True, header=True)
        self._dataview.done_layout()

    def _on_simu_step(self, evt):
        self._highlight_none()
        self.board.highlight_none()
        self._actlist.highlight_step(evt.step)

    def _on_simu_reset(self, evt):
        self._highlight_none()
        self.board.highlight_none()
        self.board.bridge.meman.free()
        self._actlist.populate(self._ctrlpan.get_main_action())

        for dev in self.board.pcidevs:
            # Unlock device selection
            dev.unlock()
            for bar in dev:
                # Unlock mask modifications
                bar.write_size_mask()
