#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""This module implements Wire widget class.

@CONSTS
    HR : indicates an horizontal line
    VR : indicates a vertical line
"""

import wx
from pcisim.core.bus import Bus
from pcisim.lib.widget.unitsize import utopix
from pcisim.lib.widget import WidgetBase
from pcisim.core.locals import WIRE_COLOR, WIRE_WRITE_COLOR,\
    WIRE_READ_COLOR, BUS_WIDTH, BUS_MARGIN, PORT_WIDTH, PORT_MARGIN
__all__ = ['Wire']

HR = 1
VR = 2

class Wire(WidgetBase):
    """A graphical object which represents a link between two components
    on the board.

    This link is logical implemented by a Port or Bus object.

    A Wire is composed by horizontal and vertical lines with ease for
    collision detection.

    @ATTRIBS
        (Port) port : port represented by the wire
        (list)(list[2]) points : coordinates of points which form the wire
        (list)(wx.Rect) rects : collision rects of the lines which compose
            the wire

        color : color of the wire when normal
        writecolor : color of the wire when written
        readcolor : color of the wire when read

        (Bool) reading : flag true if wire in read state
        (Bool) writing : flag true if wire in write state
    """

    def __init__(self, port, upos, *parts):
        """
        @PARAMS
            (Port) port : port represented by the wire
            (tuple[2]) upos : initial point expressed in units
            *parts : wire components in one of the following forms:
                | (HR, width)
                | (VR, height)

        @NOTES
            'height' and 'width' are expressed in units
        """
        self._parts = parts
        self.port = port
        self.points = None
        self.rects = None
        self.reading = False
        self.writing = False
        self.color = WIRE_COLOR
        self.writecolor = WIRE_WRITE_COLOR
        self.readcolor = WIRE_READ_COLOR

        # Do scale
        super(Wire, self).__init__(upos, usize=(0,0))

    def _doscale(self):
        """Updates points and rects required for wire display.

        @NOTES
            This widget supports position scaling but not size scaling.
        """
        ipoint = utopix(*self._upos)
        parts = map(_compose_part, self._parts)

        # Porta o Bus?
        if isinstance(self.port, Bus):
            self.width = BUS_WIDTH
            margin = BUS_MARGIN
        else:
            self.width = PORT_WIDTH
            margin = PORT_MARGIN

        # Begin
        self.rects = []
        self.points = [ipoint]

        last = list(ipoint)
        for relsize in parts:
            self.rects.append( wx.Rect(last[0]-margin, last[1]-margin,
                relsize[0]+margin*2, relsize[1]+margin*2) )
            last[0] += relsize[0]
            last[1] += relsize[1]
            self.points.append(tuple(last))

def _compose_part( (orientation, length) ):
    """Translates from orientation-length to and x-y form."""
    if orientation==HR:
        return (utopix(length), 0)
    elif orientation==VR:
        return (0, utopix(length))
    else:
        assert 0, "Unknown orientation: %s" % orientation
