#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements device configuration steps."""

import wx
from pcisim.core.simuaction import SimuAction, Step
from pcisim.lib.utils import get_hex, humansize
from pcisim.data import BAR, decode_mask
from pcisim.core.events import Message, HighlightField, InspectMemory
__all__ = ['Action_Configure_Device']

class Action_Configure_Device(SimuAction):
    """A sequence of steps to configure a PCI device."""

    def __init__(self):
        self._bar = None
        self._bdf = None
        self._bari = -1
        super(Action_Configure_Device, self).__init__("Configurazione dispositivo",
            Step("Selezione registro BAR", self.select_bar),
            Step("Abilitazione registro", self.activate_register),
            Step("Richiesta maschera di memoria", self.request_maskcode),
            Step("Lettura della richiesta", self.read_mask_request),
            Step("Scrittura maschera sul bus", self.dev_writes_mask),
            Step("Lettura maschera di memoria", self.write_mask_data),
            Step("Allocazione indirizzo di memoria", self.allocate_address),
            Step("Comunicazione indirizzo assegnato", self.write_assigned_address),
            Step("Scrittura indirizzo assegnato", self.save_assigned_address)
        )

    def on_reset(self, oldstep):
        """Action to perform on reset call."""
        if not oldstep is None:
            self._bdf = oldstep.get_return_value()
        else:
            self._bdf = None
        self._bar = BAR.BAR0-4
        self._bari = -1

    def select_bar(self):
        """Send BAR specific bdf code."""
        self._bar += 4
        self._bari += 1

        if self._bar > BAR.BAR5:
            self.done()
            return

        # Set target register
        reg = self._bdf.get_field('register')
        reg.set(self._bar)
        HighlightField(reg).fire()

        self.set_subject(self.board.cpu)
        self.config_out(self._bdf)
        Message('Analisi BAR %i...' % self._bari).fire()

    def activate_register(self):
        """Activate specified bar register"""
        self.set_subject(self.board.bridge)
        self.config_in()

    def request_maskcode(self):
        """Send a magic code to request BAR mask"""
        self.set_subject(self.board.cpu)
        self.data_out(0xFFFFFFFF)

    def read_mask_request(self):
        """Comunicate request to the device"""
        self.set_subject(self.board.bridge)
        self.data_inbus()

    def dev_writes_mask(self):
        """Write BAR mask onto the bus"""
        self.set_subject(self.board.bridge)
        mask = self.board.bridge.query_memory_mask(self._bdf)
        self.data_outbus(mask)

    def write_mask_data(self):
        """Read mask data from the bus"""
        self.set_subject(self.board.cpu)
        mask = self.data_in()
        HighlightField( mask.get_field('address') ).fire()
        return mask

    def allocate_address(self):
        """Allocate memory necessary for BAR register"""
        mask = self.get_return_value().address
        self.set_subject(self.board.cpu)
        memsize = decode_mask(mask)

        if memsize == 0:
            Message('\tBAR %i non richiede allocazione di memoria' % self._bari).fire()
            # Lock address modifications
            bar.lock_address()
            self.seek(0)
            return

        Message('\tBAR %i necessita di %s' % (self._bari, humansize(memsize))).fire()
        block = self.board.bridge.reserve_address_block(memsize)
        return block

    def write_assigned_address(self):
        """Write on the bus the assigned address"""
        self.set_subject(self.board.cpu)
        block = self.get_return_value()

        bar = self.board.pcibus.value
        bar.set_address(block.start)
        HighlightField( bar.get_field('address') ).fire()
        self.data_out(bar)
        return block

    def save_assigned_address(self):
        """Save into the BAR the assigned address"""
        self.set_subject(self.board.bridge)
        block = self.get_return_value()

        self.data_inbus()

        # This call is expensive, so call it only if performance drawback is not relevant
        ctrl = wx.GetApp().get_control_panel()
        if ctrl.get_max_step_time()*1000 < ctrl.get_interval():
            InspectMemory().fire()

        Message("\tAssegnato address range %s:%s" % (get_hex(block.start, 32), get_hex(block.end, 32)) ).fire()
        self.seek(0)

