#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements device discovering steps."""

from pcisim.core.simuaction import SimuAction, Step
from pcisim.core.events import Message, HighlightField
from pcisim.data import BDF
from .action_configure import Action_Configure_Device
__all__ = ['Action_Discover_Devices']

class Action_Discover_Devices(SimuAction):
    """A sequence of steps to discover PCI devices."""

    def __init__(self):
        self._bus = 0
        self._dev = -1
        self._func = 0
        self._bdf = None
        super(Action_Discover_Devices, self).__init__("Ricerca dispositivi",
            Step("Invio BDF tramite CONFIG_PORT", self.send_bdf),
            Step("Attivazione #IDSEL dispositivo", self.activate_idsel),
            Step("Risposta del dispositivo", self.dev_get_reply),
            Step("Lettura ID dispositivo", self.read_dev_response),
            Action_Configure_Device()
        )

    def on_reset(self, oldstep):
        """Action to perform on reset call."""
        self._bus = 0
        self._dev = -1
        self._func = 0
        self._bdf = None

    def send_bdf(self):
        """Send BDF to select a device on the bus."""
        self._dev += 1

        # Exit condition
        if self._dev > 31:
            self.done()
            return

        # Newline
        Message("").fire()

        # Send bdf code to northbridge, reading register 0x0
        self.set_subject(self.board.cpu)
        self._bdf = BDF(self._bus, self._dev, self._func, reg=0x0)
        HighlightField( self._bdf.get_field('device') ).fire()
        self.config_out(self._bdf)

        Message("Ricerca dispositivo %i,%i,%i..." % (self._bus,self._dev,self._func)).fire()

    def activate_idsel(self):
        """Activate eventual IDSEL for specified device."""
        self.set_subject(self.board.bridge)

        bdf = self.config_in()
        self.board.bridge.select_device(bdf)
        return bdf

    def dev_get_reply(self):
        """Device reply if any."""
        self.set_subject(self.board.bridge)

        bdf = self.get_return_value()
        response = self.board.bridge.get_response(bdf)
        self.data_outbus(response)

    def read_dev_response(self):
        """Read device response, if any."""
        self.set_subject(self.board.cpu)
        r00 = self.data_in()

        if r00.get_raw() == 0xFFFFFFFF:
            # Device not found
            self.seek(0)
        else:
            devid = r00.get_field('Device ID')
            HighlightField( devid ).fire()
            devid = devid.get_slot_name(devid.value)
            Message("Trovato dispositivo '%s'" % devid).fire()

        # Send value to Action_Configure_Device
        return self._bdf

def _main(app):
    act = Action_Discover_Devices()

    while not act.is_done():
        if not act.is_waiting():
            print act.stepi, act.steps[act.stepi]
        else:
            print "."
        act.step()
