#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""This module contains information about BAR structure."""

from pcisim.lib.dataword import DataWord
from pcisim.lib.datafield import DataField
from pcisim.lib.utils import humansize
__all__ = ['BAR', 'decode_mask']

def decode_mask(mask):
    """Decodes BAR mask and returns required memory size.

    @PARAMS
        (int) mask : 32 bit mask value

    @RETURN
        (int) required BAR memory
    """
    field = DataField(32)
    # Convert to unsigned value
    field.set(~(mask<<4)+1)
    return field.value

class BAR(DataWord):
    """Implements a *Base Address Register* structure.

    @ATTRIBS
        (int) barid : bar number id in range 0-5

    @CONSTS
        BAR0 : offset of BAR 0 in PCI Device Structure
        BAR1 : offset of BAR 1 in PCI Device Structure
        ...
        BAR5 : offset of BAR 5 in PCI Device Structure
    """
    # BAR registers offsets
    BAR0 = 0x10
    BAR1 = 0x14
    BAR2 = 0x18
    BAR3 = 0x1C
    BAR4 = 0x20
    BAR5 = 0x24

    def __init__(self, barid, memsize=16):
        """
        @PARAMS
            (int) barid : bar number id in range 0-5
            (int) memsize : memory size required for the bar
        """
        super(BAR, self).__init__()
        self.barid = barid
        self._mask = 0
        self._memsize = 0
        self._holdsaddr = False

        mp = self.add_field('memory mapped', 1, value=0x0, hint="Indica se il dipositivo è tipo MemoryMapped o I/O mapped", ro=True)
        mp.add_slot('Memory space', 0x0)
        mp.add_slot('Address space', 0x1)

        tp = self.add_field('type', 2, value=0x0, hint="Specifica grandezza ed allineamento in memoria del registro BAR", ro=True)
        tp.add_slot("32 bits", 0x0)
        tp.add_slot("riservato PCI 3", 0x1)
        tp.add_slot("16 bits", 0x2)
        tp.add_slot("64 bits", 0x3)

        self.add_field('prefetchable', 1, value=0, hint="Indica se la memoria è caricabile in un buffer di memoria interno prima ancora dell'uso")
        self.add_field('address', 28, hint="BAR %s: Prima dell'inizializzazione viene usato per specificare la quantità di memoria da allocare al BAR.\nSuccessivamente contiene l'indirizzo di memoria di partenza assegnato al dispositivo, allineato a 16-Bytes." % self.barid)

        # holds available memsizes
        self.memsizes = DataField(28)
        self.memsizes.add_slot(humansize(0), self.gen_size_mask(0))
        for i in xrange(4,32):
            size = 2**i
            self.memsizes.add_slot(humansize(size), self.gen_size_mask(size))
        self.lock()

        self.set_size_mask( self.gen_size_mask(memsize) )
        self.write_size_mask()

    def holds_address(self):
        """Query BAR state.

        @RETURN
            | True : this BAR has been assigned an address.
            | False : this BAR holds a memory mask
        """
        return self._holdsaddr

    def get_decoded_memsize(self):
        """Get BAR decoded memory size.

        @NOTES
            Requires previous 'set_size_mask' call.
        """
        return self._memsize

    def lock_address(self, lock=True):
        """Locks or unlocks BAR value modifications."""
        self._holdsaddr = lock
        address = self.get_field('address')
        address.set_readonly(lock)

    def set_address(self, addr):
        """Assign the base address to the BAR.

        @PARAMS
            (int) addr : 32bit base address.

        @NOTES
            This functions locks BAR value modifications.
        """
        address = self.get_field('address')
        address.set(addr>>4)
        self.lock_address(True)

    def set_size_mask(self, mask):
        """Set a memory size mask for the BAR."""
        self._mask = mask
        self._memsize = decode_mask(mask)

    def gen_size_mask(self, size):
        """Generates a size mask to hold specified size.

        @PARAMS
            (int) size : size to hold.

        @RETURN
            (int) 32bit generated size mask.
        """
        encoded = ~(size-1)
        # Perform conversions
        f = DataField(28)
        f.set(encoded>>4)
        return f.value

    def write_size_mask(self):
        """Writes current set size mask to BAR value.

        @NOTES
            This functions unlocks BAR value to future modifications.
        """
        self.lock_address(False)
        self.get_field('address').set(self._mask)
