#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a PciMemory structure."""

from pcisim.lib.dataword import DataWord
from pcisim.lib.register import RegistersBlock
from .bar import BAR

class PciMemory(RegistersBlock):
    """Implements some of the *PCI Device Structure* registers.

    Structure fields 'Vendor ID' and 'Device ID' are filled with some
    real world codes. These are fetched from the following website:
        http://www.pcidatabase.com
    """
    def __init__(self):
        super(PciMemory, self).__init__(regsize=4)

        r00 = self.new_register()
        vid = r00.add_field('Vendor ID', 16, hint="Codice del venditore del dispositivo")

        vid.add_slot("Logitech Inc.", 0x046D)
        vid.add_slot("Canon", 0x04A9)
        vid.add_slot("IBM", 0x04B3)
        vid.add_slot("Filco", 0x04D9)
        vid.add_slot("Chicony Electronics Co.", 0x04F2)
        vid.add_slot("Intel® Smart Connect Technology", 0x051D)
        vid.add_slot("Aiptek USA", 0x0553)
        vid.add_slot("Alcor Micro Corp.", 0x058f)
        vid.add_slot("Omron Corp", 0x0590)
        vid.add_slot("Apple, Inc.", 0x05ac)
        vid.add_slot("D-MAX", 0x05E1)
        vid.add_slot("SUYIN Corporation", 0x064e)
        vid.rand()

        did = r00.add_field('Device ID', 16, hint="Codice di fabbrica del dispositivo")
        did.set_filter(vid)
        #  Logitech Inc.
        did.add_slot("QuickCam Communicate", 0x08F6, cat=0x046D)
        did.add_slot("USB receiver for wireless mouse", 0xC526, cat=0x046D)
        did.add_slot("G15 Gaming Keyboard", 0xC226, cat=0x046D)
        did.add_slot("MX1000 Laser Mouse", 0xC512, cat=0x046D)
        did.add_slot("MediaPlay Cordless Mouse", 0xC50E, cat=0x046D)
        #  Canon
        did.add_slot("Canon Powershot A560 camera", 0x314D, cat=0x04A9)
        #  IBM
        did.add_slot("90889", 0x4010, cat=0x04B3)
        did.add_slot("8086", 0x401, cat=0x04B3)
        did.add_slot("Audio Controller", 0x24D5, cat=0x04B3)
        #  Filco
        did.add_slot("Filco Majestouch Tactile Touch NKRO", 0x2011, cat=0x04D9)
        did.add_slot("Pleomax Multimdia Keyboard PKB-7000X", 0x1603, cat=0x04D9)
        #  Chicony Electronics Co.
        did.add_slot("Webcam Toshiba Satellite Pro 860", 0xB307, cat=0x04F2)
        did.add_slot("Integrated Camera", 0xB217, cat=0x04F2)
        did.add_slot("Webcam Gateway NV59", 0xB044, cat=0x04F2)
        #  Intel® Smart Connect Technology
        did.add_slot("APC Back-UPS ES 700", 0x0002, cat=0x051D)
        did.add_slot("INT33A0", 0x051D, cat=0x051D)
        #  Aiptek USA
        did.add_slot("Oregon Scientific DS38xx", 0x0200, cat=0x0553)
        did.add_slot("Oregon Scientific DS38xx", 0x0201, cat=0x0553)
        did.add_slot("Oregon Scientific DS38xx", 0x0202, cat=0x0553)
        #  Alcor Micro Corp.
        did.add_slot("AM usb storage", 0x0001, cat=0x058f)
        did.add_slot("Multi Flash Reader USB Device", 0x6366, cat=0x058f)
        did.add_slot("SmartCard Reader", 0x9540, cat=0x058f)
        did.add_slot("(ASUS) Management Engine Interface", 0x8CBA, cat=0x058f)
        #  Omron Corp
        did.add_slot("hid device class blood pressure monitor", 0x0028, cat=0x0590)
        #  Apple, Inc.
        did.add_slot("Alluminium Keyboard IT USB", 0x021e, cat=0x05ac)
        did.add_slot("Apple iPhone 4s", 0x1297, cat=0x05ac)
        did.add_slot("Apple iPod", 0x1293, cat=0x05ac)
        #  D-MAX
        did.add_slot("Webcam", 0x0501, cat=0x05E1)
        did.add_slot("USB 2.0 Video Capture Controller", 0x0408, cat=0x05E1)
        #  SUYIN Corporation
        did.add_slot("Acer Crystal Eye webcam", 0xa103, cat=0x064e)
        did.add_slot("SUYIN 1.3M WebCam", 0xA219, cat=0x064e)
        did.add_slot("USB 2.0 UVC 1.3M WebCam", 0xa116, cat=0x064e)
        did.add_slot("HP TrueVision HD", 0xD217, cat=0x064e)
        did.rand()
        r00.lock()

        r04 = self.new_register()
        r04.add_field('Command', 16, hint="Imposta la modalità di risposta del dispositivo ai cicli del bus")
        r04.add_field('Status', 16, hint="Registra informazioni di stato sugli eventi del bus")
        r04.lock()

        r08 = self.new_register()
        rev = r08.add_field('Revision ID', 8, hint="Numero di revisione del dispositivo")
        rev.rand()
        r08.add_field('Prog IF', 8, hint="Specifica se il dispositivo è accessibile tramite un'interfaccia di programmazione standard")
        r08.add_field('Subclass', 8, hint="Sottoclasse di dispositivo d'appartenenza")

        cls = r08.add_field('Class Code', 8, hint="Classe di dispositivo d'appartenenza")
        cls.add_slot("Device was built prior definition of the class code field", 0x00)
        cls.add_slot("Mass Storage Controller", 0x01)
        cls.add_slot("Network Controller", 0x02)
        cls.add_slot("Display Controller", 0x03)
        cls.add_slot("Multimedia Controller", 0x04)
        cls.add_slot("Memory Controller", 0x05)
        cls.add_slot("Bridge Device", 0x06)
        cls.add_slot("Simple Communication Controllers", 0x07)
        cls.add_slot("Base System Peripherals", 0x08)
        cls.add_slot("Input Devices", 0x09)
        cls.add_slot("Docking Stations", 0x0A)
        cls.add_slot("Processors", 0x0B)
        cls.add_slot("Serial Bus Controllers", 0x0C)
        cls.add_slot("Wireless Controllers", 0x0D)
        cls.add_slot("Intelligent I/O Controllers", 0x0E)
        cls.add_slot("Satellite Communication Controllers", 0x0F)
        cls.add_slot("Encryption/Decryption Controllers", 0x10)
        cls.add_slot("Data Acquisition and Signal Processing Controllers", 0x11)
        cls.add_slot("Device does not fit any defined class", 0xFF)

        r08.lock()

        r0C = self.new_register()
        r0C.add_field('Cache Line Size', 8, hint="Grandezza di ogni linea di cache in unità da 32bits")
        r0C.add_field('Latency Timer', 8, hint="Imposta il numero massimo di cicli di accesso continuo del dispositivo al bus")
        ht = r0C.add_field('Header Type', 8, hint="Determina la struttura dei restanti bit dall'indirizzo 0x10 in poi. Inoltre, se il bit 7 è attivato, il dispositivo è multifunzione (vedi BDF)", ro=True)
        ht.add_slot('Generic Device', 0x00)
        ht.add_slot('PCI-to-PCI bridge', 0x01)
        ht.add_slot('Cardbus bridge', 0x02)

        r0C.add_field('BIST', 8, hint="Permette il monitoraggio ed il controllo di dispositivi abilitati al 'built-in self test'")
        r0C.lock()

        # BAR Registers
        for i in xrange(6):
            self.add_register(BAR(i))

    def get_header(self):
        """Get PCI Device Structure header.

        @RETURN
            (RegistersBlock) structure header.
        """
        return self[0:4]

    def get_bars(self):
        """Get PCI Device Structure BAR registers.

        @RETURN
            (RegistersBlock) structure BAR registers.
        """
        return self[4:]

    def __str__(self):
        return " <HEADER>\n%s\n <BARs>\n%s" % ( self.get_header(), self.get_bars() )
