#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""This module implements objects to describe a sequence of linked 'actions'
"""

__all__ = ['Action', 'Step']

class Action(object):
    """This class represents a list of actions, where an action is
    recursively defined as a container of other actions.

    Custom actions *must* implement the following methods:
        - 'on_reset' method to reset internal variables

    Custom actions can find these methods usefull:
        - Use get_return_value() to get previous step return value
        - Call 'seek' to move through the actions steps
        - Call done() when finished

    @ATTRIBS
        (str) name : mnemonic name for the action
        (list) steps : set of sub-actions which composes the action
        (int) stepi : index for the next step in self.steps member
    """
    def __init__(self, name, *steps):
        """
        @PARAMS
            (str) name : mnemonic name for the action
            (list) steps : set of sub-actions which composes the action
        """
        self.name = name
        self.steps = steps
        self.stepi = 0
        self._done = False              # Indicates if action has finished
        self._waiting = False           # Indicates if sub-action is running
        self._has_seek = False          # True if seek method was called
        self._oldstep = None            # Previously executed step
        self._retval = None             # Current step return value

#---------------------------I M P L E M E N T-------------------------------

    def on_reset(self, oldstep=None):
        """This method is invoked when 'reset' function is called.

        A custom action should reset its internal variables to initial state
        to be able to be executed again in the future.
        This method is also called when entering a sub-action to reset
        sub-action state. In this case 'oldstep' optional parameter is set;
        it can be used to fetch additional data from previous step by
        'get_return_value'.

        @PARAMS
            | (None) oldstep - manual 'reset' method call
            | (Action) oldstep - previously executed step
        """
        raise NotImplementedError()

    def done(self):
        """Call this when action has finished."""
        self._done = True

    def is_waiting(self):
        """Query action state.

        @RETURN
            | True : sub-action is being executed so action is stalled
            | False : action is being executed
        """
        return self._waiting

    def is_done(self):
        """Query action execution state.

        @RETURN
            | True : action has ended
        """
        return self._done

    def get_next_step(self):
        """Get the next step.

        @RETURN
            (Action) next step.

        @NOTES
            In particular, should return a 'Step' instance.
        """
        return self.steps[self.stepi].get_next_step()

    def get_action_by_id(self, actname):
        """Get action using his 'name'.

        @PARAMS
            (str) actname : 'name' attribute of the action to search

        @RETURN
            | SUCCESS : (int) action index in self.steps
            | FAILURE : (None)
        """
        for stepi in xrange(len(self.steps)):
            if self.steps[stepi].name == actname:
                return stepi
        return None

    def get_return_value(self):
        """Gets return value from previous action, if any.

        @RETURN
            | SUCCESS : return value (it could be None as well)
            | FAILURE : (None)
        """
        return self._retval

    def seek(self, stepi):
        """Set next action to perform.

        This function is very usefull for implementing loops. One could
        use an attribute as a test, call 'seek' as continue instruction and
        'done' to stop.

        @PARAMS
            (int) stepi : index of the action in self.steps
        """
        # Seek to a step
        self.stepi = stepi
        # reset sub-action
        self.steps[self.stepi].reset( self._oldstep )
        self._waiting = False
        # Set seek flag
        self._has_seek = True
        self._oldstep = self.steps[self.stepi]

    def step(self):
        """Performs a step in the action.

        @RETURN
            An optional return value can be returned by the step.

        @NOTES
            The same return value is saved and can be retrieved by
            'get_return_value' method.
        """
        if self._done is True:
            return self._retval

        # Performs the step
        self._has_seek = False
        self._retval = self.steps[self.stepi].step()

        # Se ha usato la funzione seek allora non incrementare
        if not self._has_seek:
            if self.steps[self.stepi].is_done():
                i = self.stepi + 1
                if i >= len(self.steps):
                    i = 0
                self.seek(i)
            else:
                self._waiting = True

        return self._retval

    def reset(self, oldstep=None):
        """Recursively resets all the actions to be ready to new calls.

        @PARAMS
            | (None) oldstep : this should be None when called for resetting
                the action.
            | (Step) oldstep : this should be last executed Step when called
                for entering the sub-action

        @NOTES
            This functions invokes 'on_reset' method for each action.
        """
        self.on_reset(oldstep)
        for step in self.steps:
            step.on_reset(oldstep)
            step.reset(oldstep)

        self.stepi = 0
        self._done = False
        self._waiting = False
        self._oldstep = None
        self._retval = None

    def __str__(self):
        """Shows the step name and the next step"""
        return "<Action::'%s'> : %s" % (self.name, self.steps[self.stepi])

class Step(Action):
    """This class implements a particular Action composed by a single
    function.
    """
    def __init__(self, name, func):
        """
        @PARAMS
            (str) name : mnemonic name for the step
            func : callback function to call on step
        """
        assert hasattr(func, '__call__')
        super(Step, self).__init__(name, func)

    def step(self):
        """Calls the callback function and returns optional return value."""
        self._retval = self.steps[0]()
        self.done()
        return self._retval

    def get_next_step(self):
        """Step is not a collection."""
        return self

    def reset(self, oldstep):
        """Reset function does nothin on a Step."""
        pass

    def on_reset(self, oldstep=None):
        """on_reset function does nothin on a Step."""
        pass

    def __str__(self):
        """Shows step name"""
        return "<Step::'%s'> " % self.name
