#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A module which implements a generic container for a bitfield."""

from random import randint
from . import MyException
from .utils import get_hex, get_bin
__all__ = ['DataField', 'DataFieldOverflow']

class DataField(object):
    """A bitfield container with name, bits, value and description.

    @ATTRIBS
        (str) name : symbolic name for the field
        (int) bits : number of bits of the field
        (int) value : internal value of the field
        (str) hint : field description or None

    In order to handle field value, the following interfaces are available:
        - self.value : retrieve the current field value
        - self.set : set field value making size checks and notify
            changed value to callbacks.
        - self.value = x : set field value directly (*not recommended*)

    It is possible to add callbacks to be executed on value change. This
    remarks provider-subscriber programming pattern.

    Field values can be limited using *slots*. Slots can be classified in
    category groups and, optionally, a *filter* field can be used to query
    current category.
    See 'add_slot*' and 'set_filter' methods for more details.

    @NOTES
        - A readonly field cannot be modified
        - Value is actually changed only if different from current one
    """
    def __init__(self, bits, name="", value=0, hint=None, ro=False):
        self.name = name
        self.bits = bits
        self.value = value
        self.hint = hint
        self._hot = False               # If true, field is highlighted
        self._callbacks = []            # to be called on value change
        self._readonly = ro             # read-only flag
        self._slots = {}                # filter categories
        self._filter = None             # current filter DataField

#---------------------------P U B L I C   A P I-----------------------------

    def set(self, value, notify=True):
        """Set field value after performing size checks.

        @PARAMS
            (int | long) value : value to set
            notify : if True, send value changes notifications to callbacks

        @NOTES
            This function sends callbacks notification.

        @RAISES
            DataFieldOverflow on field value overflow.
        """
        # Same value
        if self.value == value:
            return

        if not self._readonly:
            # Signed to unsigned
            if value < 0:
                value = 2**self.bits+value

            if not self.can_hold(value):
                raise DataFieldOverflow(self, value)
            #~ print "old", self.value, "new", value
            self.value = value

        if notify:
            self.notify()

    def set_readonly(self, readonly):
        """Set readonly status.

        @PARAMS
            (bool) readonly : if True, field modifications are locked
        """
        self._readonly = readonly

    def is_readonly(self):
        """Returns readonly boolean state"""
        return self._readonly

    def set_filter(self, field):
        """Adds a filter field for category filtering.

        @PARAMS
            (DataField) field : field used as a filter
        """
        # Remove previous callback
        if not self._filter is None:
            self._filter.rem_callback(self._on_filter_change)
        self._filter = field
        # Add callback to reset value
        if hasattr(self._filter, 'add_callback'):
            self._filter.add_callback(self._on_filter_change)

    def notify(self, exclude=[]):
        """This function is called to notify to the callbacks that field
        value id changed.

        @PARAMS
            | (list) exclude : optional list of callbacks to exclude
            | (func) exclude : single callback to exclude
        """
        if not hasattr(exclude, '__iter__'): exclude = (exclude, )
        [ callback(self) for callback in self._callbacks if not callback in exclude ]

    def highlight(self, highlight=True):
        """Sets field selection state.

        @PARAMS
            (bool) highlight : higlight state, True if active.

        @NOTES
            This function sends callbacks notification.
        """
        self._hot = highlight
        # Call callback
        self.notify()

    def has_highlight(self):
        """Get field selection state.

        @RETURN
            | HAS HIGHLIGHT : True
            | HAS NOT HIGHLIGHT : False
        """
        return self._hot

    def get_bin(self):
        """Get binary representation as a string."""
        return get_bin(self.value, self.bits)

    def get_hex(self):
        """Get hexadecimal representation as a string."""
        return get_hex(self.value, self.bits)

    def can_hold(self, value):
        """Performs checks on value to ensure it fits field bit size.

        @PARAMS
            (uint) value to check

        @RETURN
            | FITS OK : True
            | DON'T FIT : False
        """
        return isinstance(value, (int,long)) and (value >= 0) and (value >> self.bits == 0)

    def add_callback(self, callback):
        """Add a function listener for field value change.

        @PARAMS
            callback : callback to call on change.
        """
        self._callbacks.append(callback)

    def rem_callback(self, callback):
        """Removes the function from internal callbacks or does nothing.

        @PARAMS
            callback : function to remove

        @NOTES
            It is safe to call this function for an unsubscribed callback.
        """
        if callback in self._callbacks:
            self._callbacks.remove(callback)

    def rand(self):
        """Fills field with a random value, in respect to field bit size and
        optional slots.
        """
        if not self.has_slots():
            self.set( randint(0, 2**self.bits-1) )
        else:
            self.set_slot( randint(0, len(self.get_slots())-1) )

#--------------------------S L O T S  S T U F F-----------------------------

    def has_slots(self):
        """Query about field slots.

        @RETURN
            | HAS SLOTS : True
            | HAS NOT SLOTS : False
        """
        return len(self._slots) > 0

    def has_category(self, cat):
        """Query field slots and look for specified category.

        @PARAMS
            cat : slots category group name.

        @RETURN
            | HAS CATEGORY : True
            | HAS NOT CATEGORY : False
        """
        return self._slots.has_key(cat)

    def get_slots(self):
        """Get the field slots. If a filter is being set, use it to extract
        only the slots having the filtered category.

        @RETURN
            (list)(list[2]) : slots list of slot_label, slot_value
        """
        if not self._filter is None:
            return self._slots[self._filter.value]
        return self._slots[0x0]

    def add_slot(self, slotlb, slotval, cat=0x0):
        """Adds a slot to the field slots.

        @PARAMS
            (str) slotlb : label associated the slot
            (int) slotval : value associated to the slot
            cat : optional slot category

        @NOTES
            Default category code is 0x0
        """
        if not self.has_category(cat):
            self._slots[cat] = []
        self._slots[cat].append((slotlb, slotval))

    def add_slots(self, slots_l, cat=0x0):
        """Adds multiple slots at once, using same category.

        @PARAMS
            (list)(list[2]) : slots list of slot_label, slot_value
            cat : optional slots category

        @NOTES
            Default category code is 0x0
        """
        if not self.has_category(cat):
            self._slots[cat] = []
        self._slots[cat] = self._slots[cat] + slots_l

    def clean_slots(self):
        """Removes all slots and filter from the field"""
        self._slots = {}
        self.set_filter(None)

    def set_slot(self, sloti):
        """Sets the active slot for the current category.

        @PARAMS
            (int) sloti : index in slot category of the setting slot.

        @NOTES
            Internal value is changed and changes are notified.
        """
        self.set( self.get_slots()[sloti][1] )

    def get_slot_name(self, slotval):
        """Find a slot in current category by value.

        @PARAMS
            (int) slotval : slot value to search

        @RETURN
            | SUCCESS : (str) slot name
            | FAILURE : (None)
        """
        for slot in self.get_slots():
            if slot[1] == slotval:
                return slot[0]
        return None

#---------------------------I N T E R N A L S-------------------------------

    def _on_filter_change(self, field):
        # When category changes, reset first available value.
        if self.has_category(field.value):
            self.set_slot(0)

    def __str__(self):
        """Shows field name, bits and hex value."""
        return '%s[%i]=%s' % (self.name, self.bits, self.get_hex())

#--------------------------E X C E P T I O N S------------------------------

class DataFieldOverflow(MyException):
    """Occurrs when a value does not fit bit size.

    @ATTRIBS
        (DataField) field : field where overflow occurred.
        (int) value : value which caused the overflow.
    """
    def __init__(self, field, value):
        """
        @PARAMS
            (DataField) field : field where overflow occurred.
            (int) value : value which caused the overflow.
        """
        self.field = field
        self.value = value
        super(DataFieldOverflow, self).__init__()

    def __str__(self):
        return "DataField '%s' (%i bits) cannot hold value %s" % (self.field.name, self.field.bits, self.value)
