#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A module to group DataFields together."""

from . import MyException
from .datafield import DataField
__all__ = ['DataWord', 'FieldNotFound', 'StructureNotLocked']

class DataWord(object):
    """Provides a container for DataField objects.

    This can be used to describe bit-dependent data structures.

    @ATTRIBS
        (list) fields : list of DataFields in the word.

    @NOTES
        After building DataWord with add/add_field methods, call lock()
        to to calculate overall bitsize.
    """
    def __init__(self):
        # 'fields' attribute must be first one!
        self.fields = []
        self._locked = False
        self._bits = 0

#---------------------------P U B L I C   A P I-----------------------------

    def add(self, field):
        """Adds a field to the structure starting from bit 0.

        @PARAMS
            (DataField) field : field to add

        @NOTES
            Unlocks the structure if locked.
        """
        # Unlocks structure
        self._locked = False
        self.fields.append(field)

    def add_field(self, name, bits, *args, **kargs):
        """Creates a DataField and adds it to the structure starting from
        bit 0.

        @PARAMS
            (str) name : symbolic name for the field
            (int) bits : number of bits of the field
            *args : additional arguments to pass to DataField init
            **kargs : additional keyword arguments to pass to DataField init

        @RETURN
            (DataField) added field.

        @NOTES
            Unlocks the structure if locked.
        """
        field = DataField(bits, name, *args, **kargs)
        self.add( field )
        return field

    def lock(self):
        """Calculates structure bitsize.

        This method should be called after structure has been populated.
        After this call, bitsize can be retrieved by get_bits() call.
        """
        self._locked = True
        self._bits = self._calc_bits()

    def is_locked(self):
        """True if structure is locked;"""
        return self._locked

    def set_raw(self, rawdata):
        """Set a raw value to the structure, overwriting fields value.

        @PARAMS
            (int) rawdata : value to set
        """
        prevbits = 0
        for field in self.fields:
            val = rawdata >> prevbits & (2**field.bits-1)
            field.set(val)
            prevbits += field.bits

    def get_raw(self):
        """Get structure value as an integer value;

        @RETURN
            (int) raw fields value.
        """
        val = 0
        prevbits = 0

        for field in self.fields:
            val |= field.value << prevbits
            prevbits += field.bits
        return val

    def set_field(self, fieldname, value):
        """Sets a specific field value;

        @PARAMS
            (str) fieldname : name of the field to set
            (int) value : value to set

        @RETURN
            (DataField) modified field.
        """
        field = self.get_field(fieldname)
        field.set(value)
        return field

    def get_field(self, fieldname):
        """Get a field by field name.

        @PARAMS
            (str) fieldname : name of the field to search

        @RETURN
            (DataField) field found.

        @RAISES
            FieldNotFound when given field name does not match any.
        """
        for field in self.fields:
            if field.name == fieldname:
                return field
        raise FieldNotFound(fieldname)

    def get_bits(self):
        """Get structure overall bit size.

        @RETURN
            (int) bit size.

        @RAISES
            StructureNotLocked if lock() function was not called.
        """
        if not self._locked:
            raise StructureNotLocked(self)
        return self._bits

#---------------------------I N T E R N A L S-------------------------------

    def _calc_bits(self):
        bits = 0

        for field in self.fields:
            bits += field.bits
        return bits

    def __getattr__(self, attr):
        """Allows point notation to be used to access fields by name.

        @PARAMS
            (str) attr : field or attribute to get

        @RETURNS
            (int) field value

        @RAISES
            AttributeError if field or attribute not found
        """
        try:
            field = self.get_field(attr)
        except FieldNotFound:
            raise AttributeError("DataWord has no attribute '%s'" % attr)
        else:
            return field.value

    def __setattr__(self, attr, val):
        """Allows point notation to be used to set field value.

        @PARAMS
            (str) attr : field or attribute to be set
            (int) val : value to set
        """
        # Act like a dict
        if attr != 'fields':
            try:
                field = self.get_field(attr)
            except FieldNotFound:
                pass
            else:
                field.value = val
                return
        super(DataWord, self).__setattr__(attr, val)

    def __iter__(self):
        """Iterate over structure fields.

        @RETURN
            self.fields iterator.
        """
        # Iterate over
        return reversed(self.fields).__iter__()

    def __str__(self):
        s = " | ".join([field.name for field in reversed(self.fields)]) + "\n"

        # Calculates bit position
        pos = ['0']
        posi = 0
        for f in self.fields[:-1]:
            posi += f.bits
            pos.append(str(posi))
        pos.reverse()
        s += " | ".join(pos) + "\n"

        s += " | ".join([str(field.value) for field in reversed(self.fields)])
        return s

#--------------------------E X C E P T I O N S------------------------------

class FieldNotFound(MyException):
    """Occurs when a field name is not found in the DataWord.

    @ATTRIBS
        (str) fieldname : name of the field
    """
    def __init__(self, fieldname):
        """
        @PARAMS
            (str) fieldname : name of the field
        """
        self.fieldname = fieldname
        super(FieldNotFound, self).__init__()

    def __str__(self):
        return "Cannot find field named '%s'" % fieldname

class StructureNotLocked(MyException):
    """Occurs when a DataWord function requires 'lock' method to be called
    but it wasn't.

    @ATTRIBS
        (DataWord) data : subject structure
    """
    def __init__(self, data):
        """
        @PARAMS
            (DataWord) data : subject structure
        """
        self.data = data
        super(StructureNotLocked, self).__init__()

    def __str__(self):
        return "Structure is not locked: %s" % self.data

#-------------------------------T E S T-------------------------------------

def _main(app=None):
    word = DataWord()
    word.add_field('register', 8)
    word.add_field('function', 3)
    word.add_field('device', 5)
    word.add_field('bus', 8)
    word.add_field('reserved', 7)
    word.add_field('enable', 1)

    word.set_field('bus', 5)
    word.set_field('device', 31)
    word.set_field('function', 0)
    print word

    word.set_raw(4000000000)
    print word
    print word.get_raw()

    return 0
