#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A module to manage virtual memory."""

from . import MyException
from .utils import humansize
__all__ = ['NaturalMemoryManager', 'MemoryBlock', 'NotPowerOfTwo',
    'OutOfMemory']

class MemoryBlock(object):
    """Represents a virtual memory block.

    @ATTRIBS
        (int) start : block start virtual address
        (int) end : block end virtual address
        (int) size : block virtual size
    """
    def __init__(self, address=0, size=0):
        """
        @PARAMS
            (int) address : block start virtual address
            (int) size : block virtual size

        @NOTES
            self.end address is calculated.
        """
        self.start = address
        self.size = size
        self.end = 0
        self.calc_bounds()

    def calc_bounds(self):
        """Calculates self.end address."""
        self.end = self.start + self.size-1

class NaturalMemoryManager(object):
    """Emulates memory allocation for natural aligned addresses.

    Allocation algorithms are the following:
        a) Place a new block in first available position
        b) Relocate blocks to tide up space

    Algorithm b) is only used when free memory is not enough. Note that
    if b) fails too, OutOfMemory exception is raised.

    @ATTRIBS
        (list)(MemoryBlock) blocks : allocated memory blocks
    """
    def __init__(self, memsize):
        """
        @PARAMS
            (int) memsize : virtual memory size
        """
        self.blocks = []
        self._newaddr = 0x0
        self._memsize = memsize
        self._memfree = memsize
        self._relocated = []

#---------------------------P U B L I C   A P I-----------------------------

    def get_memsize(self):
        """Get total virtual memsize.

        @RETURN
            (int) virtual memory size
        """
        return self._memsize

    def get_memfree(self):
        """Get free virtual memsize.

        @RETURN
            (int) virtual free memory size
        """
        return self._memfree

    def get_relocated(self):
        """Get last relocated blocks.

        @RETURN
            (MemoryBlock)(listiterator) relocated blocks.
        """
        return self._relocated.__iter__()

    def free(self):
        """Frees all the allocated memory and destroyes blocks."""
        self.blocks = []
        self._newaddr = 0x0
        self._memfree = self._memsize

    def alloc(self, blksize):
        """Request allocation of a MemoryBlock.

        @PARAMS
            (int) blksize : size of the block to allocate.

        @RETURN
            (MemoryBlock) allocated block

        @RAISES
            - OutOfMemory if no memory is available.
            - NotPowerOfTwo if blksize is not naturally aligned.
        """
        # Size must be POT aligned
        NotPowerOfTwo(blksize).validate()

        # Check overall size
        if blksize > self._memfree:
            raise OutOfMemory(self)

        # Wasted space
        gap = self._newaddr % blksize
        if gap != 0:
            gap = blksize - gap
        #~ print "GAP=%i" % gap,

        # Get an aligned address
        block = MemoryBlock(self._newaddr+gap, blksize)
        self._memfree -= blksize
        self.blocks.append(block)

        if block.start+blksize > self._memsize:
            # Relocation needed
            self._relocate(exclude=block)
        else:
            self._newaddr = block.start+blksize

        return block

#---------------------------I N T E R N A L S-------------------------------

    def _relocate(self, exclude=None):
        """Relocates iternal blocks to fit memory.

        @PARAMS
            (MemoryBlock) exclude : do not add this block to relocation
                list.

        @RAISE
            OutOfMemory if no memory is available.
        """
        # Sort items by descending size
        sort = sorted(self.blocks, reverse=True, key=lambda x:x.size)
        self._newaddr = 0x0

        self._relocated = []
        for block in sort:
            # !! This check should never be True !!
            if self._newaddr+block.size > self._memsize:
                raise OutOfMemory(self)

            oldaddr = block.start
            block.start = self._newaddr
            block.calc_bounds()
            self._newaddr += block.size

            if oldaddr != block.start and not block is exclude:
                self._relocated.append((oldaddr, block))

#--------------------------E X C E P T I O N S------------------------------

class NotPowerOfTwo(MyException):
    """Indicates that given value is not a power of 2 base.

    @ATTRIBS
        (int) value : subject value
    """
    def __init__(self, value):
        """
        @PARAMS
            (int) value : subject value
        """
        self.value = value
        super(NotPowerOfTwo, self).__init__()

    def validate(self):
        """This function validates the value.

        If value is ok, no exception is raised.

        @RAISES
            NotPowerOfTwo if validation fails.
        """
        if bin(self.value-1).find('0',1) != -1:
            raise self

    def __str__(self):
        return "Value '%i' not power of 2" % self.size

class OutOfMemory(MyException):
    """Indicates that a memory manager has no free memory available.

    @ATTRIBS
        (NaturalMemoryManager) manager : subjct memory manager.
    """
    def __init__(self, manager):
        """
        @PARAMS
            (NaturalMemoryManager) manager : subjct memory manager.
        """
        self.manager = manager
        super(OutOfMemory, self).__init__()

    def __str__(self):
        return "Max memory size (%s) exceeded: only %s available" % (
            humansize(self.manager.get_memsize()),
            humansize(self.manager.get_memfree())
        )

def _main(app):
    from .utils import get_bin
    from .datafield import DataField
    from . import B, KB, MB, GB
    BITS = 5

    def print_block(block):
        print "%s:%s (%i bytes)" % (get_bin(block.start, BITS), get_bin(block.end, BITS), block.size)

    mem = NaturalMemoryManager(32*B)
    sizes = (2*B, 8*B, 16*B, 4*B, 2*B)

    for size in sizes:
        block = mem.alloc(size)
        for oldaddr,newblk in mem.get_relocated():
            print "RELOCATION from", get_bin(oldaddr, BITS), "to",
            print_block(newblk)
        print_block(block)

    print "---"
    map(print_block,mem.blocks)
