#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a RegistersBlock."""

from .dataword import DataWord, FieldNotFound
from . import MyException
__all__ = ['RegistersBlock', 'UnalignedAddress']

class RegistersBlock(object):
    """Implements a logical block of DataWords, here called registers."""
    def __init__(self, regsize, offset=0x0):
        """
        @PARAMS
            (int) regsize : size of each register
            (int) offset : initial size to skip

        @RAISES
            UnalignedAddress if offset is not alligned to regsize.
        """
        if not offset % regsize == 0:
            raise UnalignedAddress(offset, regsize)

        self._regs = []
        self.regsize = regsize
        self.offset = offset

#---------------------------P U B L I C   A P I-----------------------------

    def add_register(self, reg):
        """Adds a register to the block.

        @PARAMS
            (DataWord) reg : register to add.
        """
        self._regs.append(reg)

    def new_register(self):
        """Creates and adds new register.

        @RETURN
            (DataWord) created register.
        """
        regval = DataWord()
        self._regs.append(regval)
        return regval

    def get_register(self, offset):
        """Get a register by register offset.

        @PARAMS
            (int) offset : register offset.

        @RETURNS
            (DataWord) register.

        @RAISES
            UnalignedAddress if offset is not alligned to register size.
        """
        # Register number
        if not offset%self.regsize==0:
            raise UnalignedAddress(offset, self.regsize)
        #~ print self.offset, regno, regno/self.regsize
        return self._regs[offset/self.regsize]

    def get_field(self, field):
        """Find a field in the block by name.

        @PARAMS
            (str) field : name of the field to search

        @RETURN
            (DataField) object

        @RAISES
            FieldNotFound if no field was found.

        @NOTES
            If multiple fields share the same name, only first one is
            returned.
        """
        for reg in self._regs:
            try:
                return reg.get_field(field)
            except FieldNotFound:
                pass
        raise FieldNotFound(field)

#---------------------------I N T E R N A L S-------------------------------

    def __getitem__(self, item):
        """Get a register content or a specific field value.

        @PARAMS
            | (int) item : register position aligned to regsize
            | (str) item : DataField to find
            | (slice) item : create registers slice

        @RETURN
            | (DataWord) register if (int) was passed.
            | (DataField) field if (str) was passed.
            | (RegistersBlock) register if a (slice) was passed.

        @RAISES
            UnalignedAddress if offset is not alligned to register size.
            FieldNotFound if no field was found.
        """
        # Field search
        if isinstance(item, str):
            return self.get_field(item)
        elif isinstance(item, slice):
            # Slice registers
            ra = RegistersBlock(self.regsize, offset=item.start*self.regsize)
            regs = self._regs[item.start:item.stop]
            map(ra._regs.append, regs)
            return ra
        else:
            return self.get_register(item)

    def __setitem__(self, field, val):
        """Set a field by name.

        @PARAMS
            (str) field : field name to be set
            (int) val : field value to set

        @RETURN
            (DataField) modified field.

        @RAISES
            FieldNotFound if no field was found.
            DataFieldOverflow on field value overflow.

        @NOTES
            If multiple fields share the same name, only first one is
            returned.
        """
        field = self.get_field(field)
        field.set(val)
        return field

    def __iter__(self):
        """Get an iterator over (DataWord) registers."""
        return self._regs.__iter__()

    def __str__(self):
        lines = []

        regid = self.offset
        for reg in self:
            content = " ".join( [ "%s: %s" % (field.name, field.get_hex()) for field in reg ] )
            lines.append( "%s %s" % (hex(regid), content) )
            regid += self.regsize
        return "\n".join(lines)

class UnalignedAddress(MyException):
    """Indicates that an address is not alligned.

    @ATTRIBS
        (int) address : subject address
        (int) alignsize : aligned unit size
    """
    def __init__(self, address, alignsize):
        """
        @PARAMS
            (int) address : subject address
            (int) alignsize : aligned unit size
        """
        self.address = address
        self.alignsize = alignsize
        super(UnalignedAddress, self).__init__()

    def __str__(self):
        return "Address '%i' address must be %i units aligned" % (
            self.address, self.alignsize)
