#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A scalable grid board."""

import wx
from .unitsize import utopix, get_unit_size
from .widget import Widget
__all__ = ['GridBoard']

class GridBoard(Widget):
    """This class implements a scalable grid which is able to contain
    widgets.

    @ATTRIBS
        (bool) grid_show : if True, will draw the grid cells (default=False)
        (int) grid_width : the border width of the cells (default=1)
        grid_color : the cells border color (default='black')
    """
    def __init__(self, parent, usize, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            (list[2]) usize : widget size in units
            **kargs : keyword arguments to pass to Widget.

        @NOTES
            transparent is True by default.
            selectable is False by default.
        """
        self.grid_show = False
        self.grid_color = 'black'
        self.grid_width = 1
        kargs.setdefault('transparent', True)
        kargs.setdefault('selectable', False)
        super(GridBoard, self).__init__(parent, upos=(0,0), usize=usize,
            **kargs)

    def update_graphics(self, dc):
        """Optionally draws the grid."""
        # Optional grid
        if self.grid_show:
            # Prepare
            unit = get_unit_size()
            size = self.GetSize()

            pen = wx.Pen(self.grid_color, self.grid_width)
            dc.SetPen(pen)

            # Horizontal lines
            for j in xrange(self._usize[1]+1):
                dc.DrawLine(0, j*unit, size[0], j*unit)
            # Vertical lines
            for i in xrange(self._usize[0]+10):
                dc.DrawLine(i*unit, 0, i*unit, size[1])

    def _doscale(self):
        if self.grid_show:
            size = (utopix(self._usize[0])+self.grid_width,
                utopix(self._usize[1])+self.grid_width)
        else:
            size = utopix(*self._usize)

        # Override to fix size
        self.SetSize(size)
        self.SetMinSize(size)
        self.SetMaxSize(size)
        self.Refresh()

#-------------------------------T E S T-------------------------------------

def _build(parent):
    from wx.lib.scrolledpanel import ScrolledPanel
    from . import LabelWidget, Widget

    board = GridBoard(parent, (35,25))
    board.grid_show = True

    # Board widgets
    l = LabelWidget(board, usize=(1,1), label='Hello world')
    w = Widget(board, upos=(6,5), usize=(2,2))
    sub = Widget(w, upos=(0,1), usize=(1,1))
    w.SetBackgroundColour('red')
    sub.SetBackgroundColour('blue')

    return board
