#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""This module implements a generic visual widget consisting of a wx.Panel
instance.

@NOTES
    If you don't want to inherit from wx.Panel, consider using WidgetBase
    class instead.
"""

import wx
from .widgetbase import WidgetBase
from .unitsize import utopix
__all__ = ['Widget', 'SELECTION_COLOR', 'SELECTION_WIDTH']

SELECTION_COLOR = '#FFFF00'             # color of widgets highlight border
SELECTION_WIDTH = 4                     # width of widgets highlight border

class Widget(WidgetBase, wx.Panel):
    """A Widget consisting of a wx.Panel with an optional background color.

    When using this class as a base, take care of implementing the following
    methods:
        update_graphics(self, dc) : custom widget graphics.
        on_select(self) : custom action to take on widget selection.
    Moreover, the following events are used internally:
        wx.EVT_ERASE_BACKGROUND
        wx.EVT_PAINT
        wx.EVT_LEFT_DOWN
    an shouln't be bind by subclasses.

    Widget graphics consist of three elements:
        - background, if any.
        - custom graphics, drawn by 'update_graphics' method.
        - outline, if widget is selected.
    Call update(self, dc) method to draw all these three elements in correct
    order.

    @ATTRIBS
        sel_color : color to use for widget highlight border
        sel_width : width of the border for widget highlight

    @NOTES
        - Widget supports highlight state shown as a colored border around
          the widget. See 'set_selection' method for details.
        - Right clicking on a 'selectable' widget will call its 'on_select'
          method.

    Please use on_select() method or Skip() on EVT_LEFT_DOWN
    """
    def __init__(self, parent, upos=(0,0), usize=(1,1), selectable=True,
        transparent=False, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            (list[2]) upos : widget position in units
            (list[2]) usize : widget size in units
            (bool) selectable : enables widget selection functionality
            (bool) transparent : if False, don't use a widget background
            **kargs : wx.Panel additional parameters
        """
        wx.Panel.__init__(self, parent, **kargs)
        WidgetBase.__init__(self, upos, usize)
        self._skip_selection = not selectable
        self._selected = False
        self._transparent = transparent
        self.sel_color = SELECTION_COLOR
        self.sel_width = SELECTION_WIDTH

        # Background event handling
        if transparent is True:
            wx.Panel.Bind(self, wx.EVT_ERASE_BACKGROUND, lambda ev:None)
        self.Bind(wx.EVT_PAINT, lambda ev: self.update(wx.PaintDC(self)))
        self.Bind(wx.EVT_LEFT_DOWN, self._on_selection)

#---------------------------I M P L E M E N T-------------------------------

    def on_select(self):
        """Implements custom action to take on device selection"""
        pass

    def update_graphics(self, dc):
        """Writes graphics on the widget

        @PARAMS
            (wx.DC) dc : device context used for graphic operations.
        """
        pass

#---------------------------P U B L I C   A P I-----------------------------

    def update(self, dc):
        """Updates entire widget background, graphics and outline.

        @PARAMS
            (wx.DC) dc : device context used for graphic operations.
        """
        # Background
        if self._transparent:
            self._paint_background(dc)

        # Custom graphics
        dc.SetBrush(wx.Brush(wx.BLACK, style=wx.TRANSPARENT))
        self.update_graphics(dc)

        # Outline
        if self._selected:
            self._paint_outline(dc)

    def set_selection(self, sel=True):
        """Sets widget selection state and updates the widget.

        @PARAMS
            (bool) sel : activate or deactivate selection

        @NOTES
            This function does NOT call 'on_selection' method.
        """
        self._selected = sel
        self.Refresh()

    def GetBackgroundColour(self):
        """Retrieves background color for this widget.

        @RETURN
            | Success : background color.
            | Failure : None

        @NOTES
            If widget is transparent, returns parent background color.
        """
        if not self._transparent:
            return super(Widget, self).GetBackgroundColour()

        # Transparent Background: return parent one
        parent = self.GetParent()
        if parent is not None and hasattr(parent, 'GetBackgroundColour'):
            return parent.GetBackgroundColour()
        return None

#---------------------------I N T E R N A L S-------------------------------

    def _paint_outline(self, dc):
        # Draws an outline around the widget
        dc.SetBrush(wx.Brush(wx.BLACK, style=wx.TRANSPARENT))
        pen = wx.Pen(self.sel_color)
        pen.SetWidth(self.sel_width)
        dc.SetPen(pen)
        dc.DrawRectangle(0, 0, *self.GetSize())

    def _on_selection(self, evt=None):
        # This is called in response to a mouse click on the widget
        if self._skip_selection:
            parent = self.GetParent()
            if hasattr(parent, '_on_selection'):
                parent._on_selection()
        else:
            self.set_selection(True)
            self.on_select()

    def _doscale(self):
        self.SetPosition( utopix(*self._upos) )
        self.SetSize( utopix(*self._usize) )

    def _paint_background(self, dc):
        # only used for transparent widgets, try to inherit background.
        parent = self.GetParent()

        if hasattr(parent, 'GetSubBitmap'):
            bitmap = parent.GetSubBitmap( self.GetRect() )
        #~ elif isinstance(parent, wx.StaticBitmap):
            #~ bitmap = parent.GetBitmap()
        elif isinstance(parent, wx.Panel):
            color = parent.GetBackgroundColour()
            dc.SetBrush( wx.Brush(color) )
            dc.SetPen( wx.Pen(color) )
            dc.DrawRectangle(0, 0, *self.GetSize())
            return
        else:
            # Unknown type
            return

        # Draw bitmap on the panel
        dc.DrawBitmap(bitmap, 0, 0)

def _build(parent):
    class MyWidget(Widget):
        def on_select(self):
            print "Selected!"

    w = MyWidget(parent, usize=(3,3))
    w.SetBackgroundColour('red')
    return w
