#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""This module contains the base class of all the widgets."""

from .unitsize import get_unit_size
__all__ = ['WidgetBase']

class WidgetBase(object):
    """Abstract class which serves as a base for widget creation.

    A subclass is required to implement the following methods:
        _doscale(self) : widget scaling logic

    @NOTES
        For multi-inheritance to take place corretly, call this class
        constructor *after* any other class base.
    """
    def __init__(self, upos, usize):
        """
        @PARAMS
            (list[2]) upos : widget position in units
            (list[2]) usize : widget size in units

        @NOTES
            This constructor calls the instance _doscale() method to scale
            widget just after initialization.
        """
        self._upos = upos
        self._usize = usize
        self._doscale()

    def _doscale(self):
        raise NotImplementedError()

    def scale(self, newunit):
        """Scales the widget using an unitsize.

        @PARAMS
            | (list[2]) newunit : custom unit to use
            | (None) newunit : use default unit size

        @NOTES
            Children widgets will be scaled as well.
        """
        if newunit is None:
            newunit = get_unit_size()

        if hasattr(self, 'GetChildren'):
            # Scale children
            for child in self.GetChildren():
                if isinstance(child, WidgetBase):
                    child.scale(newunit)

        # Scale self
        self._doscale()

    def __str__(self):
        return "<%s @%x>" % (self.__class__.__name__, id(self))
