#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a StepCtrl.

@EVENTS
    EVT_CONTROL_STEP : fired when StepCtrl value changes.
        Access 'value' attribute to get new value.
"""

import wx
import wx.lib.newevent
__all__ = ['StepCtrl', 'EVT_CONTROL_STEP']

EventControlStep, EVT_CONTROL_STEP = wx.lib.newevent.NewEvent()

class StepCtrl(object):
    """An abstract class for creating a wx.Control with a limited range of
    'step' values.

    A subclass *must* implement the following methods:
        GetValue() : return control internal value
        SetValue(value) : set control internal value

    Additional methods which should be called properly are:
        check_value
        _do_fire_event
        _do_all
        _set_value
    """
    def __init__(self, step=1):
        self._last = self.GetValue()
        self._step = 1
        self.set_step(step)

#---------------------------I M P L E M E N T-------------------------------

    def GetValue(self):
        raise NotImplementedError()

    def SetValue(self, value):
        raise NotImplementedError()

#---------------------------P U B L I C   A P I-----------------------------

    def set_step(self, step):
        """Set internal step value.

        @PARAMS
            (int) step : control step value
        """
        assert step > 0
        self._step = step

    def get_step(self):
        """Get internal step value.

        @RETURN
            (int) step : control step value
        """
        return self._step

    def check_value(self, *evt):
        val = self.GetValue()
        times = val / self._step
        delta = val - self._last

        if val % self._step!=0:
            if delta > 0:
                times += 1
        self._set_value( self._step * times )

#---------------------------I N T E R N A L S-------------------------------

    def _set_value(self, val):
        """Call this to set final control value.

        @PARAMS
            (int) val : value to set

        @NOTES
            This is called automatically by 'check_value'.
        """
        self._last = val
        self.SetValue(self._last)

    def _do_fire_event(self, *evt):
        """Notifies about changed value"""
        wx.PostEvent(self, EventControlStep(value=self._last))

    def _do_all(self, *evt):
        """Checks value and Notifies about changed value"""
        self.check_value()
        self._do_fire_event()
