#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a StepSlider."""

import wx
from .stepctrl import StepCtrl, EVT_CONTROL_STEP
__all__ = ['StepSlider', 'EVT_CONTROL_STEP']

class StepSlider(wx.Slider, StepCtrl):
    """Implements a wx.Slider with step values.

    The following modes are available:
        SLIDER_FIRE_SCROLL : fire events continuosly while scrolling
        SLIDER_FIRE_END : fire events only when slide handle is released

    The following events are bound internally:
        wx.EVT_SCROLL_ENDSCROLL
        wx.EVT_MOTION
        wx.EVT_LEFT_DCLICK
        wx.EVT_LEFT_DOWN
        wx.EVT_LEFT_UP

    @EVENTS
        EVT_CONTROL_STEP on value change.

    @NOTES
        Actually only zero
    """

    SLIDER_FIRE_END = 0
    SLIDER_FIRE_SCROLL = 1

    def __init__(self, parent, mode=0, step=1, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            (int) mode : event fire mode
                SLIDER_FIRE_END | SLIDER_FIRE_SCROLL
            (int) step : control step value
            **kargs : additional keyword arguments to pass to wx.Slider
        """
        wx.Slider.__init__(self, parent, **kargs)
        StepCtrl.__init__(self, step)
        assert mode in (StepSlider.SLIDER_FIRE_SCROLL,
            StepSlider.SLIDER_FIRE_END), "Unknown slider mode: %s" % mode
        self.mode = mode

        self.Bind(wx.EVT_SCROLL_ENDSCROLL, self._do_all)
        self.Bind(wx.EVT_MOTION, self._handle_mouse_move)
        self.Bind(wx.EVT_LEFT_DCLICK, self._handle_mouse_down)
        self.Bind(wx.EVT_LEFT_DOWN, self._handle_mouse_down)
        self.Bind(wx.EVT_LEFT_UP, self._do_fire_event)

    def _get_slider_pos(self, evt):
        if self.GetWindowStyleFlag() & wx.SL_HORIZONTAL:
            # Is horizontal
            mousepos = evt.GetPosition()[0]
            size = self.GetSize()[0]
        else:
            # Is vertical
            mousepos = evt.GetPosition()[1]
            size = self.GetSize()[1]

        r = self.GetRange()
        pixstep = size / (abs(r[1]) - abs(r[0]))
        curval = r[0]+int(mousepos / pixstep)

        # Bound check
        if curval < r[0]:
            return r[0]
        elif curval >= r[1]:
            return r[1]
        return curval

    def _handle_mouse_move(self, evt):
        state = wx.GetMouseState()
        if state.LeftDown():
            # Is dragging
            slidepos = self._get_slider_pos(evt)

            if slidepos % self._step == 0:
                self._set_value(slidepos)

                if self.mode == StepSlider.SLIDER_FIRE_SCROLL:
                    self._do_fire_event()


    def _handle_mouse_down(self, evt):
        slidepos = self._get_slider_pos(evt)
        self.SetValue(slidepos)
        self.check_value()

def _build(parent):
    def callback(ev):
        print ev.value

    s = StepSlider(parent, mode=StepSlider.SLIDER_FIRE_END, size=(200,-1),
        step=20)
    s.SetRange(0, 200)
    s.Bind(EVT_CONTROL_STEP, callback)
    return s
