#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements an ActionList."""

import wx
__all__ = ['ActionList']

class ActionList(wx.Panel):
    """A widget to display an Action information."""
    def __init__(self, parent, **kargs):
        super(ActionList, self).__init__(parent, **kargs)
        self._selitem = None
        self._actree = wx.TreeCtrl(self, style=wx.TR_HAS_BUTTONS | wx.TR_FULL_ROW_HIGHLIGHT | wx.TR_SINGLE)

        l = wx.BoxSizer(wx.VERTICAL)
        l.Add(self._actree, 1, wx.EXPAND)
        self.SetSizer(l)

    def populate(self, action):
        """Creates action tree.

        @PARAMS
            (Action) action : action source.
        """
        # Cleanup
        self._actree.DeleteAllItems()
        self._selitem = None

        # Parent node
        parent = self._actree.AddRoot(action.name)
        # Indexes and return stack
        goback = []
        stepi = 0

        done = False
        while not done:
            # Discend action hierarchy
            while stepi < len(action.steps):
                step = action.steps[stepi]
                # Add element to self._actree
                elem = self._actree.AppendItem(parent, step.name)

                # If it's actually an Action container
                if len(step.steps) > 1:
                    # Enter
                    goback.append(stepi)
                    goback.append(parent)
                    goback.append(action)
                    stepi = 0
                    parent = elem
                    action = step
                else:
                    stepi += 1
            if len(goback) == 0:
                done = True
            else:
                # back tracking
                action = goback.pop()
                parent = goback.pop()
                stepi = goback.pop()+1

        # Expand root
        self._actree.Expand(parent)

    def get_item_by_label(self, label, parent=None):
        """Get an item in the tree by label string.

        @PARAMS
            (str) label : label string to search
            parent : optionally restricts search context

        @RETURN
            (TreeItemId) item

        @NOTES
            Use item.IsOk() to check if function was successful
        """
        if parent is None:
            parent = self._actree.GetRootItem()
        item, cookie = self._actree.GetFirstChild(parent)

        while item.IsOk():
            text = self._actree.GetItemText(item)
            if text == label:
                return item
            if self._actree.ItemHasChildren(item):
                match = self.get_item_by_label(label, item)
                if match.IsOk():
                    return match
            item, cookie = self._actree.GetNextChild(parent, cookie)
        return wx.TreeItemId()

    def highlight_step(self, step):
        """Marks one step into the tree.

        @PARAMS
            (Action) step to mark.
        """
        item = self.get_item_by_label(step.name)

        assert item.IsOk()

        # Deselect previous
        if not self._selitem is None:
            self._actree.SetItemTextColour(self._selitem, wx.BLACK)

            # Collapse what is not needed
            prevparent = self._actree.GetItemParent(self._selitem)
            if self._actree.GetItemParent(item) != prevparent:
                self._actree.Collapse(prevparent)

        # If it's actually an Action container
        if len(step.steps) > 1:
            self._actree.Expand(item)
        else:
            self._actree.SetItemTextColour(item, wx.RED)
            self._actree.SelectItem(item)
            self._actree.ScrollTo(item)
        self._selitem = item

def _build(parent):
    from pcisim.data import Action_Discover_Devices
    a = ActionList(parent)
    a.populate( Action_Discover_Devices() )
    return a
