#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

import time
import wx
from pcisim.lib.wxext import StepSlider, StepSpin, EVT_CONTROL_STEP
from pcisim.lib.widget.unitsize import set_unit_size, get_unit_size
from pcisim.data import Action_Discover_Devices
from pcisim.core.locals import DEBUG_STEPS, BACKGROUND_COLOR,\
    FOREGROUND_COLOR, SIMULATION_DEFAULT_INTERVAL
from pcisim.core.events import SimulatorStart, SimulatorStep,\
    SimulatorEnds, SimulatorReset
__all__ = ['ControlPanel']

class ControlPanel(wx.Panel):
    """A panel to control simulation process.

    ControlPanel records max step time. Use 'get_max_step_time' to get it.

    @NOTES
        Activate DEBUG_STEPS to show max step time it in the log panel.
    """
    def __init__(self, parent, **kargs):
        super(ControlPanel, self).__init__(parent, **kargs)
        self.SetBackgroundColour(BACKGROUND_COLOR)
        self.SetForegroundColour(FOREGROUND_COLOR)

        # Constraits
        BOX_MARGIN = 5
        BOX_SPACING = 10
        BUTTONS_SPACING = 4
        SLIDER_STEP = 4

        # Logic
        self._action = None
        self._maxstept = 0
        self._timer = wx.Timer(self)
        self._first = True

        # Widgets
        self._display = wx.TextCtrl(self, size=(400,50), style=wx.TE_LEFT |
            wx.TE_READONLY | wx.TE_MULTILINE)
        font = wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.NORMAL, False,
            'Monospace')
        self._display.SetFont(font)

        unitl = wx.StaticText(self, label="pixel/unit")
        unitw = StepSlider(self, size=(200,30), minValue=16,
            value=get_unit_size(), maxValue=64, step=SLIDER_STEP,
            style=wx.SL_HORIZONTAL | wx.SL_LABELS)
        self._interval = StepSpin(self, initial=SIMULATION_DEFAULT_INTERVAL,
            step=50, min=150, max=4000, style=wx.SP_ARROW_KEYS)
        self._interval.SetForegroundColour(wx.BLACK)
        self._btimer = wx.Button(self, label="Start")
        self._btstep = wx.Button(self, label="Step")
        self._btreset = wx.Button(self, label="Reset")

        # Bindings
        self.Bind(wx.EVT_TIMER, lambda ev: self.step())
        self._btimer.Bind(wx.EVT_BUTTON, self._on_timer_button)
        self._btstep.Bind(wx.EVT_BUTTON, lambda ev: self.step())
        self._btreset.Bind(wx.EVT_BUTTON, lambda ev: self.reset())
        unitw.Bind(EVT_CONTROL_STEP, self._on_change_unit)

        # Layout
        line1 = wx.BoxSizer(wx.HORIZONTAL)
        line2 = wx.BoxSizer(wx.HORIZONTAL)
        line3 = wx.BoxSizer(wx.HORIZONTAL)
        vert = wx.BoxSizer(wx.VERTICAL)
        horiz = wx.BoxSizer(wx.HORIZONTAL)
        box = wx.BoxSizer()

        line1.Add( wx.StaticText(self, label='Intervallo'),
            flag=wx.ALIGN_CENTER_VERTICAL )
        line1.Add(self._interval, flag=wx.LEFT|wx.RIGHT, border=10)
        line1.Add( wx.StaticText(self, label='ms'),
            flag=wx.ALIGN_CENTER_VERTICAL )

        line2.Add(self._btimer)
        line2.AddSpacer(BUTTONS_SPACING)
        line2.Add(self._btstep)
        line2.AddSpacer(BUTTONS_SPACING)
        line2.Add(self._btreset)

        line3.Add(unitl, flag=wx.TOP, border=15)
        line3.AddSpacer(10)
        line3.Add(unitw, flag=wx.EXPAND)

        # MORE Options
        more = wx.BoxSizer(wx.VERTICAL)
        more.AddStretchSpacer()
        btsave = wx.Button(self, label='Salva log')
        btsave.Bind(wx.EVT_BUTTON, lambda ev: self.savelog_dialog())
        btabout = wx.Button(self, label='About')
        btabout.Bind(wx.EVT_BUTTON, lambda ev: wx.GetApp().about_dialog())
        more.Add( btsave, flag=wx.LEFT | wx.RIGHT | wx.BOTTOM,
            border=BOX_SPACING )
        more.Add( btabout, flag=wx.LEFT | wx.RIGHT, border=BOX_SPACING )
        more.AddStretchSpacer()

        vert.Add(line1)
        vert.Add(line2, flag=wx.TOP|wx.ALIGN_CENTER, border=BOX_SPACING)
        vert.AddStretchSpacer()
        vert.Add(line3, flag=wx.TOP, border=BOX_SPACING)
        horiz.Add(vert, 2, flag=wx.EXPAND)
        horiz.Add(self._display, 3, flag=wx.EXPAND)
        horiz.Add(more, 0, flag=wx.EXPAND)

        box.Add(horiz, 1, flag=wx.EXPAND|wx.ALL, border=BOX_MARGIN)
        self.SetSizer(box)

        # Init
        self._action = Action_Discover_Devices()
        self.reset()

#---------------------------P U B L I C   A P I-----------------------------

    def is_simulation_running(self):
        """True if simulator is running in auto mode."""
        return self._btimer.GetLabel() == 'Stop'

    def show_message(self, message):
        """Shows a message in the log panel.

        @PARAMS
            (str) message : message to show
        """
        self._display.AppendText(" "+message+"\n")

    def stop(self):
        """Stops simulation."""
        if self.is_simulation_running():
            self._timer.Stop()
            self._enable_controls()

    def reset(self):
        """Reset simulation from start.

        @EVENTS
            SimulatorReset
        """
        self._maxstept = 0
        self._first = True
        self._action.reset()
        self._display.SetValue("")
        self.show_message('PCI Bus Simulator - ready')
        SimulatorReset().fire(self)
        self._enable_controls()

    def step(self):
        """Performs a step in the simulation.

        @EVENTS
            SimulatorEnds if step was last
            SimulatorStep otherwise
        """
        if self._action.is_done():
            self._timer.Stop()
            SimulatorEnds().fire(self)

            # Only reset available
            self._btimer.Enable(False)
            self._btstep.Enable(False)
            self._btreset.Enable(True)
        else:
            if self._first == True:
                # Send event
                SimulatorStart(self._action).fire(self)

            self._first = False
            step = self._action.get_next_step()
            SimulatorStep(step).fire(self)

            # Do step
            i = time.time()
            self._action.step()
            took = time.time() - i

            if DEBUG_STEPS:
                self.show_message("<%s (%.3f secs)>" %(step.name, took))
            if took > self._maxstept:
                self._maxstept = took

    def get_interval(self):
        """Get the interval time set by the user.

        @RETURN
            (int) interval value in milliseconds between to sequencial steps
        """
        return self._interval.GetValue()

    def get_main_action(self):
        """Get current main action.

        @RETURN
            (Action) current main action
        """
        return self._action

    def get_max_step_time(self):
        """Get the highest time taken for a step.

        @RETURN
            (int) milliseconds max step time

        @NOTES
            self.reset() resets this value.
        """
        return self._maxstept

    def savelog_dialog(self):
        """Open a dialog for log file saving and saves log.

        @NOTES
            Running simulation is stopped
        """
        app = wx.GetApp()
        self.stop()

        dialog = wx.FileDialog(
            parent = app.frame,
            message = "Export log file",
            wildcard = "Text files (*.txt)|*.txt;|All Files (*.*)|*",
            style = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT )

        if dialog.ShowModal() == wx.ID_OK:
            fname = dialog.GetPath()
            dialog.Destroy()

            # Actual save
            lines = "\n".join([line.strip() for line
                in self._display.GetValue().split("\n")])
            f = open(fname, "w")
            f.write(lines)
            f.close()

#---------------------------I N T E R N A L S-------------------------------

    def _enable_controls(self):
        self._btimer.SetLabel('Start')
        self._btimer.Enable(True)
        self._btstep.Enable(True)
        self._interval.Enable(True)
        self._btreset.Enable(True)

    def _on_change_unit(self, evt):
        set_unit_size(evt.value)
        board = wx.GetApp().board
        parent = board.GetParent()

        board.scale(evt.value)
        if hasattr(parent, 'SetupScrolling'):
            parent.SetupScrolling(scrollToTop=False)

    def _on_timer_button(self, evt):
        if not self.is_simulation_running():
            # Disable other controls
            self._btstep.Enable(False)
            self._interval.Enable(False)
            self._btreset.Enable(False)

            self._timer.Start(self._interval.GetValue())
            self._btimer.SetLabel('Stop')
            # First step
            self.step()
        else: # 'Stop'
            self.stop()

def _build(parent):
    c = ControlPanel(parent)
    c.Fit()
    return c
