#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""A module to visualize DataWord and RegistersBlock data in a structured
way.

It provides the following features:
    - Field values shown in hexadecimal into a table like object
    - Configurable show/hide field names/labels/bit sizes-
    - Field hints shown as ToolTips
    - Field graphical percentual size respects field bit size
"""

import wx
from wx.lib.scrolledpanel import ScrolledPanel
from pcisim.core.locals import *
from pcisim.lib.utils import get_hex
from pcisim.lib.wxext import Label
__all__ = ['DataView', 'DataViewWindow']

class DataView(wx.Panel):
    """An object to display RegistersBlock/DataWord values in a unified way.

    This class provides deep control over the internal layout.
    It is possible, for example, to manually add table contents using
    internal self.table sizer.

    Control over table rows is gained by 'new_line' and 'get_current_line'
    methods.

    A common standard is also provided by 'add_*_row' functions. They
    take a column parameter and return a column value; this can be used
    to chain multiple calls and get the desidered behaviour.

    @ATTRIBS
        (wx.GridBagSizer) table : internal view sizer.
    """
    def __init__(self, parent, bgcoloror=TABLE_BACKGROUND,
        fgcoloror=TABLE_FOREGROUND, *args, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            bgcoloror : background color of the panel
            fgcoloror : foreground color of the panel
            *args : additional wx.Panel parameters
            **kargs : additional wx.Panel keyword parameters
        """
        super(DataView, self).__init__(parent, *args)
        self._currow = 0

        kargs.setdefault('hgap', TABLE_PADDING)
        kargs.setdefault('vgap', TABLE_PADDING)
        self.table = wx.GridBagSizer(**kargs)
        self.SetSizer(self.table)

        # Background colors
        self.SetBackgroundColour(bgcoloror)
        self.SetForegroundColour(fgcoloror)

#---------------------------P U B L I C   A P I-----------------------------

    def clear(self):
        """Removes all the contents in the view."""
        self.table.Clear(deleteWindows=True)
        self._currow = 0

    def Add(self, *args, **kargs):
        """Calls internal self.table sizer Add method."""
        self.table.Add(*args, **kargs)

    def add_bitsize_row(self, fields, col=0):
        """Adds bit ranges to the current raw.

        @PARAMS
            (DataWord) fields : fields descriptor
            (int) col : initial column

        @RETURN
            (int) new column
        """
        n = fields.get_bits()
        for field in fields:
            oldn = n-1
            n -= field.bits
            hs = wx.BoxSizer(wx.HORIZONTAL)

            # not a 1-bit field
            if oldn != n:
                hs.Add( wx.StaticText(self, label=str(oldn)) )
            hs.AddStretchSpacer()
            hs.Add( wx.StaticText(self, label=str(n)) )
            span = _field_span(field)
            self.table.Add(hs , (self._currow,col), span=span,
                flag=wx.EXPAND)
            col += span[1]
        return col

    def add_fieldnames_row(self, fields, col=0):
        """Adds field names to the current row.

        @PARAMS
            (list)(DataField) fields which contain information
            (int) col : initial column

        @RETURN
            (int) new column
        """
        for field in fields:
            span = _field_span(field)
            self.table.Add( Label(self, field.name,
                bgcolor=FIELD_NAME_BACKGROUND), (self._currow, col),
                span=span, flag=wx.EXPAND )
            col += span[1]
        return col

    def add_fields(self, fields, row, col=0, names=False, slots=False):
        """Adds fields with desired options.

        @PARAMS
            (list)(DataField) fields which contain information
            (int) row : initial row
            (int) col : initial column
            (Bool) names : if True, show field names into cells
            (Bool) slots : if True, show slots as dropdown list

        @RETURN
            (int) new column

        @NOTES
            Expensive function, the more fields it has to show, the slower
            it is.
        """

        for field in fields:
            span = _field_span(field)

            if slots and field.has_slots():
                valabel = self._build_slots(field)
                callback = lambda f, label=valabel:\
                    _update_field_combo(label, f)
            else:
                valabel = Label(self, size=TABLE_CONTROLS_SIZE,
                    label=_field_label(field), bgcolor=_field_color(field))
                callback = lambda f, label=valabel:\
                        _update_field_label(label, f)
            # Bind callback
            field.add_callback(callback)
            valabel.Bind(wx.EVT_WINDOW_DESTROY, lambda ev,
                field=field,callback=callback: field.rem_callback(callback))

            # Tooltip
            if field.is_readonly():
                if field.hint is None:
                    hint = "(ro)"
                else:
                    hint = "(ro) %s" % field.hint
            else:
                hint = field.hint
            if hint is not None:
                valabel.SetToolTipString(hint)

            if names is True:
                cell = wx.BoxSizer(wx.HORIZONTAL)
                namelab = Label(self, size=TABLE_CONTROLS_SIZE,
                    label=field.name, bgcolor=FIELD_NAME_BACKGROUND)
                cell.Add(namelab , proportion=0 )
                namelab.SetToolTipString(field.name)
                cell.Add( valabel, proportion=1 )
            else:
                cell = valabel

            self.table.Add( cell, (row,col), span=span, flag=wx.EXPAND )
            col += span[1]
        return col

    def add_registers(self, registers, names=False, slots=False, bits=False,
        header=False):
        """Adds a block of registers to the view.

        @PARAMS
            registers : (RegistersBlock) registers container
            (Bool) names : if True, show field names into cells
            (Bool) slots : if True, show slots as dropdown list
            (Bool) bits : if True, show bitsize row
            (Bool) header : if True, show header containing field names

        @NOTES
            Expensive function, the more fields it has to show, the slower
            it is.
        """
        self._setup_header(registers[0], bits, header, col=1)

        # Registers
        regid = registers.offset
        j = self._currow
        for reg in registers:
            self.table.Add( wx.StaticText(self,
                label=hex(regid).upper()) , (j,0), flag=wx.ALIGN_RIGHT )
            self.add_fields(reg, row=j, col=1, names=names, slots=slots)
            regid += registers.regsize
            j += 1
        self._currow = j

    def add_word(self, dataw, names=False, slots=False, bits=False,
        header=False):
        """Adds a DataWord to the view.

        @PARAMS
            (DataWord) dataw : field container
            (Bool) names : if True, show field names into cells
            (Bool) slots : if True, show slots as dropdown list
            (Bool) bits : if True, show bitsize row
            (Bool) header : if True, show header containing field names

        @NOTES
            Expensive function, the more fields it has to show, the slower
            it is.
        """
        self._setup_header(dataw, bits, header, col=0)

        # Colonne dati
        self.add_fields(dataw, row=self._currow, col=0,
            names=names, slots=slots)
        self._currow += 1

    def get_current_line(self):
        """Returns current line number."""
        return self._currow

    def new_line(self):
        """Creates a new line and returns line number."""
        r = self._currow
        self._currow += 1
        return r

#---------------------------I N T E R N A L S-------------------------------

    def _setup_header(self, dataw, shot_bits, show_header, col):
        # Bits row
        if shot_bits is True:
            self.add_bitsize_row( dataw, col=col )
            self.new_line()

        # Field name row
        if show_header is True:
            self.add_fieldnames_row( dataw, col=col )
            self.new_line()

    def _build_slots(self, field):
        combo = wx.ComboBox(self, size=TABLE_CONTROLS_SIZE,
            style=wx.CB_READONLY)
        _update_field_combo(combo, field)

        # On change value: update field
        combo.Bind(wx.EVT_TEXT, lambda ev, combo=combo,field=field:\
            field.set_slot(combo.GetSelection()))
        return combo

class DataViewWindow(ScrolledPanel):
    """Proxy around a DataView to add margin and scrollbars.

    @NOTES
        Refer to DataView API for more details.
    """

    def __init__(self, parent, margin=4, *args, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            (int) margin : view margin in pixels
            *args : additional DataView parameters
            **kargs : additional ScrolledPanel parameters
        """
        super(DataViewWindow, self).__init__(parent, **kargs)
        dview = DataView(self, *args)
        self.__dict__['_proxy'] = dview

        self.SetBackgroundColour( dview.GetBackgroundColour() )
        self.SetForegroundColour( dview.GetForegroundColour() )

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(dview, 1, flag=wx.EXPAND|wx.ALL, border=margin )
        self.SetSizer(sizer)
        self.SetupScrolling()

    def done_layout(self):
        """Updates panel scrollbars.

        @NOTES
            Call this function when DataView layout structure is complete.
        """
        dview = self.__dict__['_proxy']
        dview.Layout()
        self.SetupScrolling()

    def __getattr__(self, attr):
        """Get DataView attributes and functions."""
        return getattr(self.__dict__['_proxy'], attr)

#---------------------------I N T E R N A L S-------------------------------

def _field_span(field):
    return (1, field.bits)

def _field_color(field):
    #Returns field value background color
    if field.has_highlight():
        color = FIELD_HIGHLIGHT_BACKGROUND
    else:
        color = FIELD_VALUE_BACKGROUND
    return color

def _field_label(field, slot=None):
    #Returns a value label for the field

    # Provided slot?
    if slot is None:
        if not field.has_slots():
            return field.get_hex()

        # Slot field
        name = field.get_slot_name(field.value)
        val = field.value
    else:
        name = slot[0]
        val = slot[1]

    if not name is None:
        return "%s | %s" % (get_hex(val, field.bits), name)
    else:
        # Fallback to normal value
        return field.get_hex()

def _update_field_label(label, field):
    label.SetValue(field.get_hex())
    label.SetBackgroundColour(_field_color(field))

def _update_field_combo(combo, field):
    combo.SetBackgroundColour(_field_color(field))
    combo.Clear()

    i = 0
    sel = None
    for slot in field.get_slots():
        combo.Append( _field_label(field, slot) )
        if field.value == slot[1]:
            sel=i
        i+=1

    # Initial value
    if not sel is None:
        combo.SetSelection(sel)
    else:
        combo.SetValue(field.get_hex())

#-------------------------------T E S T-------------------------------------

def _build(parent):
    from pcisim.data import BDF, PciMemory

    w = DataViewWindow(parent)
    w.add_word(BDF(1,2,3), bits=True, header=True)
    w.add_word(BDF(2,3,4))
    w.new_line()
    w.add_registers(PciMemory(), names=True, slots=True)
    l = w.new_line()
    txt = wx.TextCtrl(w, value='Test Control')
    w.Add(txt, (l, 0))
    print "Line:", w.get_current_line()

    w.done_layout()
    return w
