#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a Led widget."""

import wx
from pcisim.lib.widget import Widget
__all__ = ['Led']

class Led(Widget):
    """A widget class providing ON/OFF state LED."""

    def __init__(self, parent, oncolor='#00FF00', offcolor='#FF0000',
     bordercolor='#777777', *args, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            oncolor : color of the led when on
            offcolor : color of the led when off
            bordercolor : color of the led border
            *args : additional Widget class parameters
            **kargs : additional WIdget class keyword parameters
        """
        kargs.setdefault('transparent', True)
        kargs['selectable'] = False
        super(Led, self).__init__(parent, *args, **kargs)
        self._on = False
        self._onclr = oncolor
        self._offclr = offcolor
        self._bordclr = bordercolor

    def update_graphics(self, dc):
        """Draws the LED graphics."""
        super(Led, self).update_graphics(dc)

        if self.is_on():
            color = self._onclr
        else:
            color = self._offclr
        dc.SetBrush(wx.Brush(color))
        dc.SetPen(wx.Pen(self._bordclr))

        size = self.GetSize()
        ray = min(size[0]/2, size[1]/2)-1
        dc.DrawCircle(ray, ray, ray)
        del dc

    def on(self):
        """Put the led in ON state."""
        self._on = True
        self.Refresh()

    def off(self):
        """Put the led in OFF state."""
        self._on = False
        self.Refresh()

    def toggle(self):
        """Toggles the led state."""
        self._on = not self._on
        self.Refresh()

    def is_on(self):
        """Query led state.

        @RETURN
            | ON : True
            | OFF : False
        """
        return self._on is True

#-------------------------------T E S T-------------------------------------

def _build(parent):
    pan = wx.Panel(parent)
    led = Led(pan)
    on = wx.Button(pan, label='On')
    off = wx.Button(pan, label='Off')
    toggle = wx.ToggleButton(pan, label='Toggle')
    toggle.SetValue( led.is_on() )

    on.Bind(wx.EVT_BUTTON, lambda evt: led.on() or toggle.SetValue(True))
    toggle.Bind(wx.EVT_TOGGLEBUTTON, lambda evt: led.toggle())
    off.Bind(wx.EVT_BUTTON, lambda evt: led.off() or toggle.SetValue(False))

    buttons = wx.GridSizer(hgap=10, rows=1)
    buttons.Add(on)
    buttons.Add(toggle)
    buttons.Add(off)

    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(led, flag=wx.ALIGN_CENTER)
    sizer.AddSpacer(5)
    sizer.Add(buttons, flag=wx.ALIGN_CENTER)
    pan.SetSizer(sizer)
    pan.Fit()

    return pan
