#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a NorthBridge."""

import wx
from pcisim.core.bus import Port
from pcisim.lib.widget import ImageWidget
from pcisim.lib.memory import NaturalMemoryManager
from pcisim.core.locals import VIRTUAL_MEMORY_SIZE, pickimage
from pcisim.data import BDF, RawData
from pcisim.core.events import InspectMemory
from pcisim.core.events import AddressRelocation
from pcisim.core.wirewidget import WireWidget
__all__ = ['NorthBridge']

class NorthBridge(WireWidget, ImageWidget):
    """Represents and emulates northbridge functionalities.

    @ATTRIBS
        (NaturalMemoryManager) meman : memory manager
        (Port) port_config : configuration port linked to the cpu
        (Port) port_data : data port linked to the cpu

    @NOTES
        Use 'set_bridge_link' cpu method to enstablish link with cpu.
    """
    def __init__(self, parent, bus, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            (Bus) bus : the PCI bus instance
            **kargs : additional ImageWidget keyword arguments
        """
        kargs.setdefault('usize', (3,3))
        kargs['imgpath'] = pickimage('northbridge.png')
        WireWidget.__init__(self)
        ImageWidget.__init__(self, parent, **kargs)

        # Memory manager
        self.meman = NaturalMemoryManager(VIRTUAL_MEMORY_SIZE)

        # Expose links
        self.pcibus = bus
        self.port_config = Port(32, 'CONFIG_PORT')
        self.port_data = Port(32, 'DATA_PORT')

    def on_select(self):
        """Custom Widget action: show memory dump."""
        InspectMemory().fire(self)

    def query_memory_mask(self, bdf):
        """Ask current device for memory mask.

        @PARAMS
            (BDF) bdf : bdf code containing BAR register

        @RETURN
            (int) 28 bits device memory mask
        """
        self.pcibus.link.write_memory_mask(bdf.register)
        mask = self.get_response(bdf)
        return mask

    def select_device(self, bdf):
        """Analize BDF code and select device, if exists, on the bus.

        @PARAMS
            (BDF) bdf : bdf code describing device location.
        """

        # deselect current device
        if self.pcibus.link is not None:
            self.pcibus.link.unselect()
        pcidev = self.pcibus.set_link(bdf.device)

        # try to select new one
        if not pcidev is None:
             pcidev.select()

    def get_response(self, bdf):
        """Get register data from a device.

        @PARAMS
            (BDF) bdf : bdf code containing register number

        @NOTES
            Assumes 'select_device' has been called before
        """
        if self.pcibus.link is None:
            response = 0xFFFFFFFF
        else:
            # Inform device about data
            response = self.pcibus.link.get_regdata(bdf.register)
        return response

    def reserve_address_block(self, blksize):
        """Use internal memory manager to reserve an address block.

        @PARAMS
            (int) blksize : size of the block to reserve

        @RETURN
            (MemoryBlock) Allocated block

        @EVENTS
            AddressRelocation if occurred
        """
        blk = self.meman.alloc(blksize)

        # Notify relocations
        for oldaddr, newblock in self.meman.get_relocated():
            AddressRelocation(oldaddr, newblock).fire(self)
        return blk

def _build(parent):
    board = wx.GetApp().board
    nb = NorthBridge(parent, board.pcibus)
    return nb
