#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements PCIConfiguration configuration dialog."""

import wx
from pcisim.lib.utils import humansize
from pcisim.lib.datafield import DataField
from pcisim.data import decode_mask
from .dataview import DataView
__all__ = ['PCIConfiguration']

class PCIConfiguration(wx.Dialog):
    """A Dialog to access PCI devices configuration.

    It provides a DataView interface to allow field specific modifications.

    Use 'load_device_info' method to feed the configurator a PCI device
    configuration. It is possible to call this multiple times without
    closing the dialog window.
    """
    def __init__(self, parent, **kargs):
        """
        @PARAMS
            (wx.Window) parent : window parent
            **kargs : additional keyword arguments for wx.Dialog
        """
        kargs.setdefault('title', '256 bytes Configuration space')
        super(PCIConfiguration, self).__init__(parent, **kargs)
        self._dev = None
        self._limits = [DataField(28, value=0x0) for i in xrange(6)]

        self._memview = DataView(self)
        self.SetBackgroundColour( self._memview.GetBackgroundColour() )
        self.SetForegroundColour( self._memview.GetForegroundColour() )

        # Widget to show available memory
        self._freew = wx.StaticText(self)

        # Label to show device info
        self._pcilabel = wx.StaticText(self)
        font = self._pcilabel.GetFont()
        font.SetPointSize(14)
        font.SetWeight(wx.BOLD)
        self._pcilabel.SetFont(font)

        # Controls
        self._prevd = wx.Button(self, label='<')
        self._prevd.SetToolTipString("Dispositivo precedente")
        self._nextd = wx.Button(self, label='>')
        self._nextd.SetToolTipString("Dispositivo successivo")
        self._setup_controls()
        hs = wx.BoxSizer(wx.HORIZONTAL)
        hs.Add( self._prevd )
        hs.AddStretchSpacer()
        hs.Add( self._pcilabel )
        hs.AddStretchSpacer()
        hs.Add( self._nextd )

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add( hs, flag=wx.EXPAND | wx.ALL, border=20 )
        sizer.Add(self._memview, flag=wx.ALL, border=10)
        sizer.Add(self._freew, flag=wx.ALIGN_RIGHT | wx.BOTTOM | wx.RIGHT,
            border=10)
        self.SetSizer(sizer)

        self.Bind(wx.EVT_WINDOW_DESTROY, self._clean_bar_fields )

#---------------------------P U B L I C   A P I-----------------------------

    def load_device_info(self, pcidev):
        """Loads a device configuration info into the view.

        @PARAMS
            (PciDeviceice) pcidev : the PCI widget from which to load infoes
        """
        # Cleanup and new setup
        self._clean_bar_fields()
        self._memview.clear()
        self._dev = pcidev
        self._setup_controls()
        self._pcilabel.SetLabel( "Pci device #%i" % pcidev.get_device_id() )

        # BAR memory limitations
        self._setup_bar_fields()
        self._reset_bar_limits()

        # Add Header
        pcimem = pcidev.get_device_memory()
        self._memview.add_registers( pcimem.get_header(), slots=True,
            bits=True, names=True )
        self._memview.new_line()
        j = self._memview.new_line()
        self._memview.Add( wx.StaticText(self._memview,
            label='BAR registers'), (j, 0), span=(1,33),
            flag=wx.ALIGN_CENTER )

        # Add BARs
        self._memview.add_registers(pcimem.get_bars(), slots=True,
            bits=True, header=True )
        j = self._memview.new_line()
        self._memview.Add( wx.StaticText(self._memview, label='...'),
            (j, 0), flag=wx.ALIGN_RIGHT)

        self.GetSizer().Layout()
        self.Fit()

#---------------------------I N T E R N A L S-------------------------------

    def _get_free_memory(self):
        # Computes max free memory available.
        board = wx.GetApp().board
        freemem = board.bridge.meman.get_memsize()

        for dev in board.pcidevs:
            for bar in dev:
                freemem -= bar.get_decoded_memsize()

        # Update label
        self._freew.SetLabel("%s disponibili per l'allocazione" %
            humansize(freemem))
        # Reposition the label
        self.GetSizer().Layout()
        return freemem

    def _setup_controls(self):
        # Setup the right and left arrows links to move back and forth
        if self._dev is None:
            self._prevd.Enable(False)
            self._nextd.Enable(False)
            return

        pcidevs = wx.GetApp().board.pcidevs
        i = pcidevs.index(self._dev)
        if i == 0:
            # Can't go back
            self._prevd.Enable(False)
        else:
            self._prevd.Enable(True)
            self._prevd.Bind( wx.EVT_BUTTON, lambda ev, self=self,
                dev=pcidevs[i-1]: self.load_device_info(dev) )
        if i >= len(pcidevs)-1:
            # Can't go on
            self._nextd.Enable(False)
        else:
            self._nextd.Bind( wx.EVT_BUTTON, lambda ev, self=self,
                dev=pcidevs[i+1]: self.load_device_info(dev) )
            self._nextd.Enable(True)

    def _setup_bar_fields(self):
        # Initializes fields hooks for further control
        b = 0
        for bar in self._dev:
            # Do not touch assigned BARs
            if bar.holds_address():
                b+=1
                continue

            address = bar.get_field('address')
            sizes = bar.memsizes.get_slots()

            # Create limit categories
            address.add_slots(sizes, cat=0)
            for i in xrange(1, len(sizes)):
                # Create category
                address.add_slots(sizes[:-i], cat=i)

            # Setup limit
            address.add_callback( self._reset_bar_limits )
            # HACK : use a control field as a filter for bars memory sizes
            # 0: All
            # 1: All-1
            # 2: All-2
            # n-1: 1
            address.set_filter( self._limits[b] )
            b += 1

    def _reset_bar_limits(self, address=None):
        # Limits BAR selectable memory size not to overflow the avail. mem.

        # update size mask
        for bar in self._dev:
            if not bar.holds_address():
                bar.set_size_mask(bar.address)

        memfree = self._get_free_memory()
        i = 0
        # Set per-bar limit
        for bar in self._dev:
            # Case address already assigned no slots
            if bar.holds_address():
                i += 1
                continue

            # Do not notify, otherwise this callback will recursively called
            self._limits[i].set(0, notify=False)

            # Extract sizes column
            memsizes = [decode_mask(slot[1]) for slot in
                bar.get_field('address').get_slots()]
            barmem = bar.get_decoded_memsize()

            # Find limits
            limit = 0
            for l in xrange(0, len(memsizes)):
                if memfree+barmem < memsizes[l]:
                    #~ print memfree+barmem, memsizes[l], l
                    limit = len(memsizes)-l
                    break

            # Update the field
            self._limits[i].value = limit
            bar.get_field('address').notify(exclude=self._reset_bar_limits)
            i+=1

    def _clean_bar_fields(self, ev=None):
        # Removes the hooks used for BAR size limit.
        if not self._dev is None:
            for bar in self._dev:
                address = bar.get_field('address')
                address.clean_slots()
                address.rem_callback(self._reset_bar_limits)

def _main(app):
    conf = PCIConfiguration(None)
    conf.load_device_info(wx.GetApp().board.pcidevs[0])
    conf.ShowModal()
    conf.Destroy()
    return 0
