#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a PciDevice widget."""

import wx
from pcisim.lib.widget import ImageWidget, LabelWidget, SpinWidget
from pcisim.core.wirewidget import WireWidget
from pcisim.core.locals import pickimage
from pcisim.data import PciMemory
from pcisim.core.events import InspectDevice, OpenConfigNotification
from .pciconf import PCIConfiguration
from . import Led
__all__ = ['PciDevice']

class PciDevice(WireWidget, ImageWidget):
    """A widget which represent a PCI device.

    PciDevice holds a 'device ID', which corresponds to the bus line it
    is connected to. See 'get_device_id' and 'set_device_id' for more info.
    """
    def __init__(self, parent, devid, upos=(0,0), **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            (int) devid : device ID for bus enumeration
            (list[2]) upos : device position expressed in units
            **kargs : additional ImageWidget keyword arguments
        """
        kargs.setdefault('usize', (5,3))
        kargs['upos'] = upos
        kargs['imgpath'] = pickimage('pcidev.png')
        WireWidget.__init__(self)
        ImageWidget.__init__(self, parent, **kargs)

        # Internals
        self._memory = PciMemory()
        self._idsel = False
        self._devid = devid

        # Id indicator
        self._devidw = LabelWidget(self, upos=(0.25,0.5), usize=(0.7,0.7))
        self._devidw.SetForegroundColour('#111111')
        font = self._devidw.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        self._devidw.SetFont(font)
        self._spin = SpinWidget(self, upos=(4.4,0.5), usize=(0.4,0.8))
        self._spin.SetMax(31)
        self._spin.Bind(wx.EVT_SPIN, self._on_spin)

        # Selection indicator
        self._seled = Led(self, upos=(4.25,1.75), usize=(0.5,0.5))
        self.unselect()

        # Inspect indicator
        insp = ImageWidget(self, pickimage('settings.png'),
            upos=(0.25,1.75), usize=(0.5,0.5))
        insp.Bind( wx.EVT_LEFT_DOWN, lambda ev: self.open_config() )

        self.set_device_id(devid)

#---------------------------P U B L I C   A P I-----------------------------

    def get_device_id(self):
        """Returns device associated ID."""
        return self._devid

    def get_device_memory(self):
        """Returns a (PciMemory) instance holding device configuration."""
        return self._memory

    def get_regdata(self, reg):
        """Get device register data.

        @PARAMS
            reg : (int) register id to read

        @RETURN
            (DataWord) given register content
        """
        return self._memory.get_register(reg)

    def write_memory_mask(self, bar):
        """Writes to BAR register its internal size mask.

        @PARAMS
            (int) bar : bar register offset
        """
        bar = self._memory.get_register(bar)
        bar.write_size_mask()

    def on_select(self):
        """Action to take on selection: dump memory to DataView.

        @EVENTS
            InspectDevice
        """
        InspectDevice(self).fire(self)

    def open_config(self):
        """Open configuration dialog for this device.

        @EVENTS
            OpenConfigNotification
        """
        OpenConfigNotification().fire(self)
        dialog = PCIConfiguration(self.GetParent())
        dialog.load_device_info(self)
        dialog.CenterOnScreen()
        dialog.ShowModal()
        dialog.Destroy()

    def set_device_id(self, devid):
        """Set/change device associated ID and reposition device in the
        PCI bus.

        @PARAMS
            (int) devid : device ID for bus enumeration
        """
        self._devid = devid
        self._devidw.SetLabel( 'PCI dev #%i' % devid )
        self._spin.SetValue(devid)

        # Update bus
        bus = wx.GetApp().board.pcibus
        bus.set_slot(self, devid)

    def lock(self):
        """Locks device ID modifications from GUI."""
        self._spin.Enable(False)

    def unlock(self):
        """Unlocks device ID modifications from GUI."""
        self._spin.Enable(True)

    def select(self):
        """Activate IDSEL# line on device.

        @NOTES
            This cause 'lock' to be called.
        """
        self._idsel = True
        self._seled.on()
        self.lock()

    def unselect(self):
        """Deactivate IDSEL# line on the device."""
        self._idsel = False
        self._seled.off()

#---------------------------I N T E R N A L S-------------------------------

    def _on_spin(self, evt):
        # Drives the spin control to skip assigned device IDs
        bus = wx.GetApp().board.pcibus

        # Determine delta value
        if self._spin.GetValue()-self._devid > 0:
            delta = 1
        else:
            delta = -1

        # Find a free slot
        newid = self._devid+delta
        while newid>=0 and newid<=self._spin.GetMax():
            curdev = bus.get_slot(newid)

            # Free slot found ?
            if curdev is None:
                # Free old one
                bus.set_slot(None, self._devid)
                self.set_device_id(newid)
                # Update spin value
                self._spin.SetValue(newid)
                return

            newid += delta
        # Reset spin value
        self._spin.SetValue(self._devid)

    def __iter__(self):
        """Simplifies iteration on the device BAR registers.

        @RETURN
            (listiterator) over device BAR registrers.
        Ease BARs iteration
        """
        return self._memory.get_bars().__iter__()

def _build(parent):
    dev = PciDevice(parent, 31)
    wx.GetApp().board.pcidevs.append(dev)
    return dev
