#!/usr/bin/env python
#
# Emanuele Faranda                                 <black.silver@hotmail.it>
#
#.....................P C I  B U S  S I M U L A T O R.......................
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#...........................................................................
#

"""Implements a WireBoard."""

import wx
from . import Cpu, PciDevice, NorthBridge
from pcisim.core.bus import Bus
from pcisim.lib.widget import GridBoard
from pcisim.core.wire import HR, VR, Wire
from pcisim.lib.widget import Widget, LabelWidget
from pcisim.core.events import InspectPort
from pcisim.lib.widget.widget import SELECTION_COLOR
from pcisim.core.locals import BOARD_USIZE, BOARD_SHOW_GRID,\
    BOARD_GRID_COLOR, BOARD_GRID_WIDTH
__all__ = ['WireBoard']

class WireBoard(GridBoard):
    """Implements the simulator grid board used to place widgets.

    This provides highlight* methods to set focus on a Widget at a time.
    Please note that in order to allow (Widget) highlight features, it is
    required the widget to be subclassed from the following class:
        pcisim.core.wirewidget.WireWidget

    It also provide interaction with wires and widgets for right click.

    @ATTRIBS
        (Bool) grid_show : if True, show cells in the grid
        grid_color : color of the grid cell borders
        grod_width : width of the grid cell borders
        (CPU) cpu : the board CPU
        (NorthBridge) bridge : the board Northbridge
        (list)(PciDevice) pcidevs : list of board PCI devices
        (Bus) pcibus : board PCI bus

    @NOTES
        - Remember to call 'populate' before using the board.
    """
    def __init__(self, parent, **kargs):
        """
        @PARAMS
            (wx.Window) parent : parent window
            **kargs : additional keyword arguments

        @NOTES
            Set BOARD_USIZE in pcisim.core.locals to define board size.
        """
        super(WireBoard, self).__init__(parent, usize=BOARD_USIZE, **kargs)

        # Theme
        self.grid_show = BOARD_SHOW_GRID
        self.grid_color = BOARD_GRID_COLOR
        self.grid_width = BOARD_GRID_WIDTH
        self._selitem = None                   # Selected wire
        self._wires = []                        # Internal wires

        self.Bind(wx.EVT_LEFT_DOWN, self._on_click)

#---------------------------P U B L I C   A P I-----------------------------

    def scale(self, newunit):
        """Scales the board to a new unit.

        @PARAMS
            (list[2]) newunit : new unit size
        """
        # Update wires
        for wire in self._wires:
            wire._doscale()

        # Update self
        super(WireBoard, self).scale(newunit)

    def update_graphics(self, dc):
        """Update board graphics."""
        super(WireBoard, self).update_graphics(dc)

        # Wires
        for wire in self._wires:
            self._update_wire(wire, dc)

    def populate(self):
        """Creates widgets in the board."""

        # Components
        self.cpu = Cpu(self, (4,2))

        # BUS : 32 pci devices
        self.pcibus = Bus(bits=32, slots=32)
        self.pcidevs = [
            PciDevice(self, 0, (1,11)),
            PciDevice(self, 1, (7,11)),
            PciDevice(self, 3, (13,11)),
            PciDevice(self, 8, (19,11)),
            PciDevice(self, 16, (25,11))
        ]

        self.bridge = NorthBridge(self, self.pcibus, upos=(14,2))
        self.cpu.set_bridge_link(self.bridge)

        # Wires
        self._wires = [
            Wire(self.bridge.port_config, (7,3), (HR,7) ),
            Wire(self.bridge.port_data, (7,4), (HR,7) ),
            Wire(self.pcibus, (1,9),
                (HR, 2), (VR, 2), (VR, -2),         # dev1
                (HR, 6), (VR, 2), (VR, -2),         # dev2
                (HR, 6), (VR, 2), (VR, -2),         # dev3
                          (VR, -4), (VR, 4),        # NorthBridge
                (HR, 6), (VR, 2), (VR, -2),         # dev4
                (HR, 6), (VR, 2), (VR, -2),         # dev5
                (HR, 2) ),
        ]

        # Labels
        LabelWidget(self, label='CONFIG_PORT', upos=(8,2.4))
        LabelWidget(self, label='DATA_PORT', upos=(8,3.4))
        LabelWidget(self, label='PCI_BUS', upos=(16,8))

    def highlight(self, port, read):
        """Highlights a port.

        @PARAMS
            (Port) port : port to highlight
            (Bool) read : if True, read state, else write state
        """
        wire = self._wire_by_port(port)
        if read:
            wire.reading = True
        else:
            wire.writing = True
        self._update_wire(wire)

    def highlight_widget(self, widget):
        """Highlights a widget.

        @PARAMS
            (Widget) widget to highlight
        """
        if widget is self._selitem:
            return

        self._reset_selection()
        self._selitem = widget
        widget.set_selection(True)


    def highlight_none(self):
        """Clears all the highlighted items."""
        self._reset_selection()

        for wire in self._wires:
            wire.reading = False
            wire.writing = False
            self._update_wire(wire)

#---------------------------I N T E R N A L S-------------------------------

    def _wire_by_port(self, port):
        # Get a wire by port
        for wire in self._wires:
            if wire.port is port:
                return wire
        assert 0, "Wire for port '%s' not found" % port

    def _reset_selection(self):
        # Removes current selection
        w = self._selitem
        self._selitem = None
        if isinstance(w, Widget):
            w.set_selection(False)
        elif isinstance(w, Wire):
            self._update_wire(w)

    def _on_widget_click(self, widget):
        # Selects the widget and trigger its action
        self._reset_selection()
        self._selitem = widget
        widget.set_selection(True)
        widget.on_select()

    def _on_click(self, evt):
        # Click onto the board: look for wire collision
        pos = evt.GetPosition()
        item = None

        for wire in self._wires:
            for rect in wire.rects:
                if rect.ContainsXY(*pos):
                    item = wire
                    break
        # Same wire
        if item is self._selitem:
            return
        self._reset_selection()
        self._selitem = item

        if not item is None:
            # Send inspection event
            InspectPort(item.port).fire(self)
            self._update_wire(item)

    def _update_wire(self, wire, dc=None):
        # Updates a wire graphics
        if dc is None:
            dc = wx.ClientDC(self)

        # select color
        if self._selitem is wire:
            color = SELECTION_COLOR
        elif wire.reading:
            color = wire.readcolor
        elif wire.writing:
            color = wire.writecolor
        else:
            color = wire.color

        # Pen setup
        pen = wx.Pen(color, wire.width)
        pen.SetCap(wx.CAP_BUTT)
        pen.SetJoin(wx.JOIN_MITER)
        dc.SetPen(pen)
        dc.DrawLines(wire.points, 0,0)

def _build(parent):
    b = WireBoard(parent)
    b.populate()
    return b
